#!/usr/bin/env python
#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import datetime
import unittest

from lxml import etree

from soaplib.core import namespaces

from soaplib.core.model.clazz import Array
from soaplib.core.model.primitive import Boolean
from soaplib.core.model.primitive import DateTime
from soaplib.core.model.primitive import Float
from soaplib.core.model.primitive import Integer
from soaplib.core.model.base import Null
from soaplib.core.model.primitive import String

ns_test = 'test_namespace'

class TestPrimitive(unittest.TestCase):
    def test_string(self):
        s = String()
        element = etree.Element('test')
        String.to_parent_element('value', ns_test, element)
        element=element[0]
        
        self.assertEquals(element.text, 'value')
        value = String.from_xml(element)
        self.assertEquals(value, 'value')

    def test_datetime(self):
        d = DateTime()
        n = datetime.datetime.now()

        element = etree.Element('test')
        DateTime.to_parent_element(n, ns_test, element)
        element = element[0]

        self.assertEquals(element.text, n.isoformat())
        dt = DateTime.from_xml(element)
        self.assertEquals(n, dt)

    def test_utcdatetime(self):
        datestring = '2007-05-15T13:40:44Z'
        e = etree.Element('test')
        e.text = datestring

        dt = DateTime.from_xml(e)

        self.assertEquals(dt.year, 2007)
        self.assertEquals(dt.month, 5)
        self.assertEquals(dt.day, 15)

        datestring = '2007-05-15T13:40:44.003Z'
        e = etree.Element('test')
        e.text = datestring

        dt = DateTime.from_xml(e)

        self.assertEquals(dt.year, 2007)
        self.assertEquals(dt.month, 5)
        self.assertEquals(dt.day, 15)

    def test_integer(self):
        i = 12
        integer = Integer()

        element = etree.Element('test')
        Integer.to_parent_element(i, ns_test, element)
        element = element[0]

        self.assertEquals(element.text, '12')
        value = integer.from_xml(element)
        self.assertEquals(value, i)

    def test_large_integer(self):
        i = 128375873458473
        integer = Integer()

        element = etree.Element('test')
        Integer.to_parent_element(i, ns_test, element)
        element = element[0]

        self.assertEquals(element.text, '128375873458473')
        value = integer.from_xml(element)
        self.assertEquals(value, i)

    def test_float(self):
        f = 1.22255645


        element = etree.Element('test')
        Float.to_parent_element(f, ns_test, element)
        element = element[0]

        self.assertEquals(element.text, '1.22255645')

        f2 = Float.from_xml(element)
        self.assertEquals(f2, f)

    def test_array(self):
        type = Array(String)
        type.resolve_namespace(type,"zbank")

        values = ['a', 'b', 'c', 'd', 'e', 'f']

        element = etree.Element('test')
        type.to_parent_element(values, ns_test, element)
        element = element[0]

        self.assertEquals(len(values), len(element.getchildren()))

        values2 = type.from_xml(element)
        self.assertEquals(values[3], values2[3])

    def test_array_empty(self):
        type = Array(String)
        type.resolve_namespace(type,"zbank")

        values = []

        element = etree.Element('test')
        type.to_parent_element(values, ns_test, element)
        element = element[0]

        self.assertEquals(len(values), len(element.getchildren()))

        values2 = type.from_xml(element)
        self.assertEquals(len(values2), 0)

    def test_unicode(self):
        s = u'\x34\x55\x65\x34'
        self.assertEquals(4, len(s))
        element = etree.Element('test')
        String.to_parent_element(s, 'test_ns', element)
        element = element[0]
        value = String.from_xml(element)
        self.assertEquals(value, s)

    def test_null(self):
        element = etree.Element('test')
        Null.to_parent_element('doesnt matter', ns_test, element)
        element = element[0]
        self.assertTrue( bool(element.get('{%s}nil' % namespaces.ns_xsi)) )
        value = Null.from_xml(element)
        self.assertEquals(None, value)

    def test_boolean(self):
        b = etree.Element('test')
        Boolean.to_parent_element(True, ns_test, b)
        b = b[0]
        self.assertEquals('true', b.text)

        b = etree.Element('test')
        Boolean.to_parent_element(0, ns_test, b)
        b = b[0]
        self.assertEquals('false', b.text)

        b = etree.Element('test')
        Boolean.to_parent_element(1, ns_test, b)
        b = b[0]
        self.assertEquals('true', b.text)

        b = Boolean.from_xml(b)
        self.assertEquals(b, True)

        b = etree.Element('test')
        Boolean.to_parent_element(False, ns_test, b)
        b = b[0]
        self.assertEquals('false', b.text)

        b = Boolean.from_xml(b)
        self.assertEquals(b, False)

        b = etree.Element('test')
        Boolean.to_parent_element(None, ns_test, b)
        b = b[0]
        self.assertEquals('true', b.get('{%s}nil' % namespaces.ns_xsi))

        b = Boolean.from_xml(b)
        self.assertEquals(b, None)

if __name__ == '__main__':
    unittest.main()




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
from lxml import etree

from soaplib.core import namespaces


logger = logging.getLogger("soaplib.core.wsdl")

class WSDL():

    """ A standalone wsdl """

    def __init__(self, soap_application, ns_tns, url, with_partner_link=False):


        self.nsmap = dict(namespaces.const_nsmap)
        self.prefmap = dict(namespaces.const_prefmap)
        self.ns_tns = ns_tns
        self.pref_tns = 'tns'
        self.application = soap_application
        self.application.url = url.replace('.wsdl', '')
        self.url = self.application.url
        self._with_plink = with_partner_link
        self.service_node_dict = {}
        self.bindings_dict = {}
        self.port_type_dict = {}
        self.soap_bindings_dict = {}
        self.service_port_dict = {}

        self.elements= None


    def to_string(self,
                  pretty_print=False,
                  xml_declaration=True,
                  encoding="UTF-8"
    ):
        """Returns a string reprensation of the  wsdl as fornatted xml."""

        if self.elements is None:
            self.build_wsdl()

        return etree.tostring(
            self.elements,
            encoding=encoding,
            xml_declaration=xml_declaration,
            pretty_print=pretty_print
        )


    def _build_root(self):
        ''' Build ths root element of the wsdl '''

        self.application.set_namespace_prefix(self.ns_tns, self.pref_tns)
        self.root = etree.Element(
            "{%s}definitions" % namespaces.ns_wsdl,
            nsmap=self.application.nsmap
        )

        self.root.set('targetNamespace', self.ns_tns)
        self.root.set('name', self.application.name)


    def _build_types(self):
        ''' Builds the wsdl:types elemement  '''

        self.application.types = etree.SubElement(
            self.root,
            "{%s}types" % namespaces.ns_wsdl
        )

        self.application.build_schema(self.application.types)


    def build_messages(self):
        ''' Builds the wsdl:message elements '''

        self.application.messages = set()

        for s in self.application.services:
           s = self.application.get_service(s,None)
           s.add_messages_for_methods(
               self.application,
               self.root,
               self.application.messages
           )


    def _build_plink(self, service_name):
        ''' Builds partnerlinks '''

        self.plink = etree.SubElement(
            self.root,
            '{%s}partnerLinkType' % namespaces.ns_plink
        )

        self.plink.set('name', service_name)
        self.application.__add_partner_link(
            self.root,
            service_name,
            self.application.types,
            self.application.url,
            self.plink
        )


    def _get_or_create_service_node(self, service_name):
        ''' Builds a wsdl:service element. '''

        ser = None
        if service_name not in self.service_node_dict:

            ser = etree.SubElement(self.root, '{%s}service' % namespaces.ns_wsdl)
            ser.set('name', service_name)
            self.service_node_dict[service_name] = ser

        else:
            ser = self.service_node_dict[service_name]

        return ser


    def _add_port_to_service(self, service, port_name, binding_name):
        """ Builds a wsdl:port for a service and binding"""

        pref_tns = self.application.get_namespace_prefix(
            self.application.get_tns()
        )

        wsdl_port = etree.SubElement(service, '{%s}port' % namespaces.ns_wsdl)
        wsdl_port.set('name', port_name)
        wsdl_port.set('binding', '%s:%s' % (pref_tns, binding_name))

        addr = etree.SubElement(wsdl_port, '{%s}address' % namespaces.ns_soap)
        addr.set('location', self.application.url)


    def _get_or_create_port_type(self, pt_name):
        """ Creates a wsdl:portType element. """

        pt = None

        if pt_name not in self.port_type_dict:

            pt = etree.SubElement(self.root, '{%s}portType' % namespaces.ns_wsdl)
            pt.set('name', pt_name)
            self.port_type_dict[pt_name] = pt
        else:
            pt = self.port_type_dict[pt_name]

        return pt

    def _get_or_create_binding(self, binding_name, port_type_name):
        ''' Creates a wsdl:binding element for a portType. '''

        binding = None
        if binding_name not in self.bindings_dict:
            binding = etree.SubElement(
                self.root,
                '{%s}binding' % namespaces.ns_wsdl
            )

            binding.set('name', binding_name)
            binding.set('type', '%s:%s'% (self.pref_tns, port_type_name))
            self.bindings_dict[binding_name] = binding
        else:
            binding = self.bindings_dict[binding_name]

        return binding

    def _get_or_create_soap_binding(self, binding):
        """ Creates a soap:binding element for a binding. """
        binding_name = binding.get('name')
        soap_binding = None

        if binding_name not in self.soap_bindings_dict:
            soap_binding = etree.SubElement(
                binding,
                '{%s}binding' % namespaces.ns_soap
            )

            soap_binding.set('style', 'document')
            soap_binding.set('transport', self.application.transport)
            self.soap_bindings_dict[binding_name] = soap_binding
        else:
            soap_binding = self.soap_bindings_dict[binding_name]

        return soap_binding

    def _get_port_name(self, port_type_name):
        return port_type_name # subclasses override to control port names.

    def _get_binding_name(self, port_type_name):
        return port_type_name # subclasses override to control port names.

    def _build_entry_points(self):
        """ Iterates over the soaplib service objects in the application and
            constructs the wsdl elements used for representing operations.
        """

        default_service_name = self.application.get_name()
        applied_service_name = None
        port_binding_names = []
        for service in self.application.services:
            ser = None
            cb_binding = None

            if service.get_service_interface() is None:
                # This is the default behavior. i.e. no service interface is
                # defined in the service heading
                applied_service_name = default_service_name
            else:
                applied_service_name = service.get_service_interface()

            port_type_list = service.get_port_types()
            if len(port_type_list) != 0:

                pt = None
                for port_type in port_type_list:

                    pt = self._get_or_create_port_type(port_type)
                    pt_name = pt.get('name')
                    binding_name = self._get_binding_name(pt_name)
                    port_name = self._get_port_name(pt_name)
                    port_binding_names.append((port_name, binding_name))
                    binding = self._get_or_create_binding(binding_name, pt_name)
                    soap_binding = self._get_or_create_soap_binding(binding)

                    s=self.application.get_service(service)
                    s.add_port_type(
                        self.application,
                        self.root,
                        applied_service_name,
                        self.application.types,
                        self.application.url,
                        pt
                    )

                    cb_binding = s.add_bindings_for_methods(
                        self.application,
                        self.root,
                        service.get_service_class_name(),
                        self.application.types,
                        self.application.url,
                        binding,
                        cb_binding
                    )

            else:

                #TODO: this needs to be extracted since it is duplicate logic.
                pt = self._get_or_create_port_type(applied_service_name)
                pt_name = pt.get('name')
                port_name = self._get_port_name(pt_name)
                binding_name = self._get_binding_name(pt_name)
                port_binding_names.append((port_name, binding_name))
                binding = self._get_or_create_binding(binding_name, pt_name)
                soap_binding = self._get_or_create_soap_binding(binding)
                s=self.application.get_service(service)
                s.add_port_type(
                    self.application,
                    self.root,
                    applied_service_name,
                    self.application.types,
                    self.application.url,
                    pt
                )

                cb_binding = s.add_bindings_for_methods(
                    self.application,
                    self.root,
                    service.get_service_class_name(),
                    self.application.types,
                    self.application.url,
                    binding,
                    cb_binding
                )

        ser = self._get_or_create_service_node(applied_service_name)
        for port_name, binding_name in port_binding_names:
            self._add_port_to_service(ser, port_name, binding_name)


    def build_wsdl(self):
        ''' Construct the elements for the wsdl. '''

        self._build_root()
        self._build_types()
        self.build_messages()


        if self._with_plink:
            self._build_plink(self.application.name)

        self._build_entry_points()
        self.elements = self.root
        return self.root



import uuid
from django.db import models
from django.utils.translation import ugettext_lazy as _
from pcart_catalog.models import (
    Collection,
    ProductType,
    Product,
    ProductVariant,
    ProductImage,
)


class XMLFile(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    filename = models.CharField(_('File name'), max_length=255)
    added = models.DateTimeField(_('Added'), auto_now_add=True)

    class Meta:
        verbose_name = _('XML import file')
        verbose_name_plural = _('XML import files')

    def __str__(self):
        return self.filename

    def run_import_task(self):
        from .tasks import import_goods
        # Execute import tasks in 30 seconds from now
        return import_goods.apply_async(args=[self.pk], countdown=30)


class ProductImportLogMessage(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    product = models.ForeignKey(
        Product, verbose_name=_('Product'), related_name='import_log_messages')
    variant = models.ForeignKey(
        ProductVariant, verbose_name=_('Product variant'),
        related_name='import_log_messages', null=True, blank=True)

    file = models.ForeignKey(
        XMLFile, verbose_name=_('XML import file'),
        related_name='import_log_messages')
    message = models.TextField(_('Message'), blank=True, default='')
    added = models.DateTimeField(_('Added'), auto_now_add=True)

    class Meta:
        verbose_name = _('Product import log message')
        verbose_name_plural = _('Product import log messages')

    def __str__(self):
        return self.message


class ProductTypeRelation(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(_("Title"), max_length=255)
    alternate_uuid = models.UUIDField(
        _('UUID in 1C'),
        unique=True,
        help_text=_('UUID of the product category in 1C system.'))
    product_type = models.OneToOneField(
        ProductType,
        verbose_name=_('Product type'),
        related_name='product_type_relation')
    collection = models.OneToOneField(
        Collection,
        verbose_name=_('Collection'), related_name='product_type_relation',
        help_text=_('Add the products of specified type to a collection.'),
    )

    class Meta:
        verbose_name = _('Product type relation')
        verbose_name_plural = _('Product type relations')

    def __str__(self):
        return self.title


class ExtraCollectionRelation(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    tag_name = models.CharField(_("Tag name"), max_length=70)
    collection = models.OneToOneField(
        Collection,
        verbose_name=_('Collection'),
        related_name='extra_collection_relation',
        help_text=_('Link MetaContent/* tag with a collection.'),
    )

    class Meta:
        verbose_name = _('Extra collection relation')
        verbose_name_plural = _('Extra collections relations')

    def __str__(self):
        return self.tag_name



from soaplib.core.service import DefinitionBase, soap
from soaplib.core.model.primitive import String, Integer

class DemoWebService(DefinitionBase):
    '''
    The demo webservice class.
    This defines methods exposed to clients.
    '''
    def __init__(self, environ):
        '''
        This saves a reference to the request environment on the current instance
        '''
        self.environ = environ
        super(DemoWebService, self).__init__(environ)

    @soap(String, _returns=String)# Soap is typed - we need stuff like this
    def echo(self, text):
        return text




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logger = logging.getLogger(__name__)


from lxml import etree

from soaplib.core import namespaces, styles
from soaplib.core import MethodDescriptor
from soaplib.core.model.clazz import ClassModel as Message
from soaplib.core.model.clazz import ClassModelMeta as MessageMeta
from soaplib.core.model.clazz import TypeInfo
from soaplib.core.model.primitive import Any

_pref_wsa = namespaces.const_prefmap[namespaces.ns_wsa]

def _produce_input_message(ns, f, params, kparams):
    _in_message = kparams.get('_in_message', f.__name__)
    _in_variable_names = kparams.get('_in_variable_names', {})
    arg_count = f.__code__.co_argcount
    param_names = f.__code__.co_varnames[1:arg_count]

    try:
        in_params = TypeInfo()

        for i in range(len(params)):
            e0 = _in_variable_names.get(param_names[i], param_names[i])
            e1 = params[i]

            in_params[e0] = e1

    except IndexError as e:
        raise Exception("%s has parameter numbers mismatching" % f.__name__)

    message=Message.produce(type_name=_in_message, namespace=ns,
                                            members=in_params)
    message.__namespace__ = ns
    message.resolve_namespace(message, ns)

    return message

def _produce_rpc_output_message(ns, f, params, kparams):
    _returns = kparams.get('_returns')

    _out_message = kparams.get('_out_message', '%sResponse' % f.__name__)

    out_params = TypeInfo()

    if _returns:
        if isinstance(_returns, (list, tuple)):
            default_names = ['%sResult%d' % (f.__name__, i) for i in
                                                           range(len(_returns))]

            _out_variable_names = kparams.get('_out_variable_names',
                                                                default_names)

            assert (len(_returns) == len(_out_variable_names))

            var_pair = zip(_out_variable_names,_returns)
            out_params = TypeInfo(var_pair)

        else:
            _out_variable_name = kparams.get('_out_variable_name',
                                                       '%sResult' % f.__name__)

            out_params[_out_variable_name] = _returns

    message=Message.produce(type_name=_out_message, namespace=ns,
                                                             members=out_params)
    message.__namespace__ = ns
    message.resolve_namespace(message, ns)

    return message


class Alias(Message):
    """New type_name, same type_info.
    """
    @classmethod
    def add_to_schema(cls, schema_dict):
        if not schema_dict.has_class(cls._target):
            cls._target.add_to_schema(schema_dict)
        element = etree.Element('{%s}element' % namespaces.ns_xsd)
        element.set('name',cls.get_type_name())
        element.set('type',cls._target.get_type_name_ns(schema_dict.app))

        schema_dict.add_element(cls, element)

def _makeAlias(type_name, namespace, target):
    """ Return an alias class for the given target class.

    This function is a variation on 'ClassModel.produce'.

    The alias will borrow the target's typeinfo.
    """
    cls_dict = {}

    cls_dict['__namespace__'] = namespace
    cls_dict['__type_name__'] = type_name
    cls_dict['_type_info'] = getattr(target, '_type_info', ())
    cls_dict['_target'] = target

    return MessageMeta(type_name, (Alias,), cls_dict)

def _produce_document_output_message(ns, f, params, kparams):
    """Generate an output message for "document"-style API methods.

    This message is just an alias for the declared return type.
    """
    _returns = kparams.get('_returns', Any)
    _out_message = kparams.get('_out_message', '%sResponse' % f.__name__)

    message = _makeAlias(_out_message, ns, _returns)
    message.resolve_namespace(message, ns)

    return message

def soap(*params, **kparams):
    '''
    This is a method decorator to flag a method as a remote procedure call.  It
    will behave like a normal python method on a class, and will only behave
    differently when the keyword '_method_descriptor' is passed in, returning a
    'MethodDescriptor' object.  This decorator does none of the soap/xml
    serialization, only flags a method as a soap method.  This decorator should
    only be used on member methods of an instance of ServiceBase.
    '''

    def explain(f):
        def explain_method(*args, **kwargs):
            retval = None

            if '_method_descriptor' not in kwargs :
                retval = f(*args, **kwargs)

            else:
                _is_callback = kparams.get('_is_callback', False)
                _public_name = kparams.get('_public_name', f.__name__)
                _is_async = kparams.get('_is_async', False)
                _mtom = kparams.get('_mtom', False)
                _in_header = kparams.get('_in_header', None)
                _out_header = kparams.get('_out_header', None)
                _port_type = kparams.get('_port_type', None)
                _style = kparams.get('_style', styles.RPC_STYLE)

                # the decorator function does not have a reference to the
                # class and needs to be passed in
                ns = kwargs['clazz'].get_tns()


                in_message = _produce_input_message(ns, f, params, kparams)

                if _style == styles.RPC_STYLE or _style is None:
                    out_message = _produce_rpc_output_message(
                            ns,
                            f,
                            params,
                            kparams
                    )

                elif _style == styles.DOC_STYLE:
                    out_message = _produce_document_output_message(ns, f, params, kparams)

                else:
                    raise ValueError(
                        """Invalid style: valid values are
                        soaplib.core.styles.RPC_STYLE or
                        soaplib.core.styles.DOC_STYLE
                        """
                    )




                _faults = kparams.get('_faults', [])

                if _in_header :
                    _in_header.resolve_namespace(_in_header, ns)
                if _out_header :
                    _out_header.resolve_namespace(_out_header, ns)

                doc = getattr(f, '__doc__')
                retval = MethodDescriptor(f.__name__,
                                          _public_name,
                                          in_message,
                                          out_message,
                                          doc,
                                          _is_callback,
                                          _is_async,
                                          _mtom,
                                          _in_header,
                                          _out_header,
                                          _faults,
                                          _style,
                                          _port_type,
                                         )
            return retval

        explain_method.__doc__ = f.__doc__
        explain_method._is_rpc = True
        # import pdb; pdb.set_trace()
        explain_method.func_name = f.__name__#func_name

        return explain_method

    return explain


def rpc(*params, **kparams):
    """This is a method decorator to flag a method as a remote procedure call.
    It will behave like a normal python method on a class, and will only behave
    differently when the keyword '_method_descriptor' is passed in, returning a
    'MethodDescriptor' object.  This decorator does none of the soap/xml
    serialization, only flags a method as a soap method.  This decorator should
    only be used on member methods of an instance of ServiceBase.

    Moving forward, this method is being depricated in favor of @soap
    Presently it simply calls @soap after checking for the _style keyword
    argument.  If the _style argument is not supplied it defaults to
    soaplib.core.styles.RPC_STYLE
    """

    if "_style" not in kparams:
        kparams["_style"] = styles.RPC_STYLE

    return soap(*params, **kparams)





def document(*params, **kparams):
    """Method decorator to flag a method as a document-style operation.

    It will behave like a normal python method on a class, and will only
    behave differently when the keyword '_method_descriptor' is passed in,
    returning a 'MethodDescriptor' object.

    This decorator does none of the soap/xml serialization, only flags a
    method as a soap method.  This decorator should only be used on member
    methods of an instance of a class derived from 'ServiceBase'.

    Moving forward, this method is being depricated in favor of @soap
    Presently it simply calls @soap after setting _style keyword
    argument to  soaplib.core.DOC_STYLE
    """

    kparams["_style"] = styles.DOC_STYLE

    return soap(*params, **kparams)



_public_methods_cache = {}

class DefinitionBase(object):
    '''
    This class serves as the base for all soap services.  Subclasses of this
    class will use the rpc decorator to flag methods to be exposed via soap.
    This class is responsible for generating the wsdl for this service
    definition.

    It is a natural abstract base class, because it's of no use without any
    method definitions, hence the 'Base' suffix in the name.
    '''

    __tns__ = None
    __in_header__ = None
    __out_header__ = None
    __service_interface__ = None
    __port_types__ = ()

    def __init__(self, environ=None):
        self.in_header = None
        self.out_header = None

        cls = self.__class__
        if not (cls in _public_methods_cache):
            _public_methods_cache[cls] = self.build_public_methods()

        self.public_methods = _public_methods_cache[cls]
        self.service_interface = cls.__service_interface__
        self.port_types = cls.__port_types__
        self.environ = environ

    @classmethod
    def get_service_class_name(cls):
        return cls.__name__

    @classmethod
    def get_service_interface(cls):
        return cls.__service_interface__

    @classmethod
    def get_port_types(cls):
        return cls.__port_types__

    def on_method_call(self, method_name, py_params, soap_params):
        '''Called BEFORE the service implementing the functionality is called

        @param the method name
        @param the tuple of python params being passed to the method
        @param the soap elements for each argument
        '''

    def on_method_return_object(self, py_results):
        '''Called AFTER the service implementing the functionality is called,
        with native return object as argument

        @param the python results from the method
        '''

    def on_method_return_xml(self, soap_results):
        '''Called AFTER the service implementing the functionality is called,
        with native return object serialized to Element objects as argument.

        @param the xml element containing the return value(s) from the method
        '''

    def on_method_exception_object(self, exc):
        '''Called BEFORE the exception is serialized, when an error occurs
        during execution.

        @param the exception object
        '''

    def on_method_exception_xml(self, fault_xml):
        '''Called AFTER the exception is serialized, when an error occurs
        during execution.

        @param the xml element containing the exception object serialized to a
        soap fault
        '''

    def call_wrapper(self, call, params):
        '''Called in place of the original method call.

        @param the original method call
        @param the arguments to the call
        '''
        return call(*params)

    @classmethod
    def get_tns(cls):
        if cls.__tns__ :
            return cls.__tns__

        service_name = cls.__name__.split('.')[-1]

        retval = '.'.join((cls.__module__, service_name))
        if cls.__module__ == '__main__':
            retval = '.'.join((service_name, service_name))

        return retval

    def build_public_methods(self):
        '''Returns a list of method descriptors for this object'''

        logger.debug('building public methods')
        public_methods = []

        for func_name in dir(self):
            if func_name == 'public_methods':
                continue
            func = getattr(self, func_name)
            if callable(func) and hasattr(func, '_is_rpc'):
                descriptor = func(_method_descriptor=True, clazz=self.__class__)
                public_methods.append(descriptor)

        return public_methods

    def get_method(self, name):
        '''Returns the metod descriptor based on element name or soap action.'''

        for method in self.public_methods:
            type_name = method.in_message.get_type_name()
            if '{%s}%s' % (self.get_tns(), type_name) == name:
                return method

        for method in self.public_methods:
            if method.public_name == name:
                return method

        raise Exception('Method "%s" not found' % name)

    def _has_callbacks(self):
        '''Determines if this object has callback methods or not.'''

        for method in self.public_methods:
            if method.is_callback:
                return True

        return False

    def header_objects(self):
        return []

    def get_service_names(self):
        '''Returns the service name(s) for this service. If this
        object has callbacks, then a second service is declared in
        the wsdl for those callbacks.
        '''

        service_name = self.__class__.__name__.split('.')[-1]

        if self._hasCallbacks():
            return [service_name, '%sCallback' % service_name]

        return [service_name]

    def add_port_type(self, app, root, service_name, types, url, port_type):
        ns_wsdl = namespaces.ns_wsdl

        # FIXME: I don't think this call is working.
        cb_port_type = self.__add_callbacks(root, types, service_name, url)

        port_name = port_type.get('name')
        method_port_type = None

        for method in self.public_methods:

            if len(self.port_types) is 0 and method_port_type is None:
                method_port_type = port_name
            else:
                method_port_type = method.port_type

            if method.is_callback:
                operation = etree.SubElement(cb_port_type, '{%s}operation'
                                                            % ns_wsdl)
            else:
                operation = etree.SubElement(port_type,'{%s}operation'% ns_wsdl)

            operation.set('name', method.name)

            if method.doc is not None:
                documentation = etree.SubElement(operation, '{%s}documentation'
                                                                % ns_wsdl)
                documentation.text = method.doc

            operation.set('parameterOrder', method.in_message.get_type_name())

            op_input = etree.SubElement(operation, '{%s}input' % ns_wsdl)
            op_input.set('name', method.in_message.get_type_name())
            op_input.set('message', method.in_message.get_type_name_ns(app))

            if (not method.is_callback) and (not method.is_async):
                op_output = etree.SubElement(operation, '{%s}output' %  ns_wsdl)
                op_output.set('name', method.out_message.get_type_name())
                op_output.set('message', method.out_message.get_type_name_ns(
                                                                           app))

                for f in method.faults:
                    fault = etree.SubElement(operation, '{%s}fault' %  ns_wsdl)
                    fault.set('name', f.get_type_name())
                    fault.set('message', f.get_type_name())

    # FIXME: I don't think this is working.
    def __add_callbacks(self, root, types, service_name, url):
        ns_xsd = namespaces.ns_xsd
        ns_wsa = namespaces.ns_wsa
        ns_wsdl = namespaces.ns_wsdl
        ns_soap = namespaces.ns_soap

        ns_tns = self.get_tns()
        pref_tns = 'tns'

        cb_port_type = None

        # add necessary async headers
        # WS-Addressing -> RelatesTo ReplyTo MessageID
        # callback porttype
        if self._has_callbacks():
            wsa_schema = etree.SubElement(types, "{%s}schema" % ns_xsd)
            wsa_schema.set("targetNamespace", '%sCallback'  % ns_tns)
            wsa_schema.set("elementFormDefault", "qualified")

            import_ = etree.SubElement(wsa_schema, "{%s}import" % ns_xsd)
            import_.set("namespace", ns_wsa)
            import_.set("schemaLocation", ns_wsa)

            relt_message = etree.SubElement(root, '{%s}message' % ns_wsdl)
            relt_message.set('name', 'RelatesToHeader')
            relt_part = etree.SubElement(relt_message, '{%s}part' % ns_wsdl)
            relt_part.set('name', 'RelatesTo')
            relt_part.set('element', '%s:RelatesTo' % _pref_wsa)

            reply_message = etree.SubElement(root, '{%s}message' % ns_wsdl)
            reply_message.set('name', 'ReplyToHeader')
            reply_part = etree.SubElement(reply_message, '{%s}part' % ns_wsdl)
            reply_part.set('name', 'ReplyTo')
            reply_part.set('element', '%s:ReplyTo' % _pref_wsa)

            id_header = etree.SubElement(root, '{%s}message' % ns_wsdl)
            id_header.set('name', 'MessageIDHeader')
            id_part = etree.SubElement(id_header, '{%s}part' % ns_wsdl)
            id_part.set('name', 'MessageID')
            id_part.set('element', '%s:MessageID' % _pref_wsa)

            # make portTypes
            cb_port_type = etree.SubElement(root, '{%s}portType' % ns_wsdl)
            cb_port_type.set('name', '%sCallback' % service_name)

            cb_service_name = '%sCallback' % service_name

            cb_service = etree.SubElement(root, '{%s}service' % ns_wsdl)
            cb_service.set('name', cb_service_name)

            cb_wsdl_port = etree.SubElement(cb_service, '{%s}port' % ns_wsdl)
            cb_wsdl_port.set('name', cb_service_name)
            cb_wsdl_port.set('binding', '%s:%s' % (pref_tns, cb_service_name))

            cb_address = etree.SubElement(cb_wsdl_port, '{%s}address'
                                                              % ns_soap)
            cb_address.set('location', url)

        return cb_port_type

    def add_schema(self, schema_entries):
        '''
        A method for adding the appropriate entries
        to the schema for the types in the specified methods.

        @param the schema node to add the schema elements to. if it is None,
               the schema nodes are returned inside a dictionary
        @param the schema node dictinary, where keys are prefixes of the schema
               stored schema node
        '''

        if self.__in_header__ != None:
            self.__in_header__.resolve_namespace(self.__in_header__,
                                                                self.get_tns())
            self.__in_header__.add_to_schema(schema_entries)

        if self.__out_header__ != None:
            self.__out_header__.resolve_namespace(self.__out_header__,
                                                                self.get_tns())
            self.__out_header__.add_to_schema(schema_entries)

        for method in self.public_methods:
            method.in_message.add_to_schema(schema_entries)
            method.out_message.add_to_schema(schema_entries)

            for fault in method.faults:
                fault.add_to_schema(schema_entries)

            if method.in_header is None:
                method.in_header = self.__in_header__
            else:
                method.in_header.add_to_schema(schema_entries)

            if method.out_header is None:
                method.out_header = self.__out_header__
            else:
                method.out_header.add_to_schema(schema_entries)

    def __add_message_for_object(self, app, root, messages, obj):
        if obj != None and not (obj.get_type_name() in messages):
            messages.add(obj.get_type_name())

            message = etree.SubElement(root, '{%s}message' % namespaces.ns_wsdl)
            message.set('name', obj.get_type_name())

            part = etree.SubElement(message, '{%s}part' % namespaces.ns_wsdl)
            part.set('name', obj.get_type_name())
            part.set('element', obj.get_type_name_ns(app))

    def add_messages_for_methods(self, app, root, messages):
        '''
        A private method for adding message elements to the wsdl
        @param the the root element of the wsdl
        '''

        for method in self.public_methods:
            self.__add_message_for_object(app,root,messages,method.in_message)
            self.__add_message_for_object(app,root,messages,method.out_message)
            self.__add_message_for_object(app,root,messages,method.in_header)
            self.__add_message_for_object(app,root,messages,method.out_header)
            for fault in method.faults:
                self.__add_message_for_object(app,root,messages,fault)


    def check_method_port(self, method):

        if len(self.port_types) != 0 and method.port_type is None:
            raise ValueError(
                """
                A port must be declared in the RPC decorator if the service
                class declares a list of ports

                Method:    %r
                """ % method.name
            )

        if (not method.port_type is None) and len(self.port_types) == 0:
            raise ValueError(
                """
                The rpc decorator has decared a port while the service class
                has not.  Remove the port declaration from the rpc decorator
                or add a list of ports to the service class
                """
            )
        try:
            if (not method.port_type is None):
                index = self.port_types.index(method.port_type)
        except ValueError as e:
            raise ValueError(
                """
                The port specified in the rpc decorator does not match any of
                the ports defined by the service class
                """
            )



    def add_bindings_for_methods(self, app, root, service_name, types, url,
                                        binding, transport, cb_binding=None):
        '''
        A private method for adding bindings to the wsdl

        @param the root element of the wsdl
        @param the name of this service
        '''

        ns_wsdl = namespaces.ns_wsdl
        ns_soap = namespaces.ns_soap
        pref_tns = app.get_namespace_prefix(self.get_tns())

        if self._has_callbacks():
            if cb_binding is None:
                cb_binding = etree.SubElement(root, '{%s}binding' % ns_wsdl)
                cb_binding.set('name', '%sCallback' % service_name)
                cb_binding.set('type', 'typens:%sCallback' % service_name)

            soap_binding = etree.SubElement(cb_binding, '{%s}binding' % ns_soap)
            soap_binding.set('transport', transport)

        for method in self.public_methods:

            self.check_method_port(method)

            operation = etree.Element('{%s}operation' % ns_wsdl)
            operation.set('name', method.name)

            soap_operation = etree.SubElement(operation, '{%s}operation' %
                                                                       ns_soap)
            soap_operation.set('soapAction', method.public_name)
            soap_operation.set('style', 'document')

            # get input
            input = etree.SubElement(operation, '{%s}input' % ns_wsdl)
            input.set('name', method.in_message.get_type_name())

            soap_body = etree.SubElement(input, '{%s}body' % ns_soap)
            soap_body.set('use', 'literal')

            # get input soap header
            in_header = method.in_header
            if in_header is None:
                in_header = self.__in_header__

            if in_header :
                soap_header = etree.SubElement(input, '{%s}header' % ns_soap)
                soap_header.set('use', 'literal')
                soap_header.set('message', in_header.get_type_name_ns(app))
                soap_header.set('part', in_header.get_type_name())

            if not (method.is_async or method.is_callback):
                output = etree.SubElement(operation, '{%s}output' % ns_wsdl)
                output.set('name', method.out_message.get_type_name())

                soap_body = etree.SubElement(output, '{%s}body' % ns_soap)
                soap_body.set('use', 'literal')

                # get input soap header
                out_header = method.in_header
                if out_header is None:
                    out_header = self.__in_header__

                if out_header:
                    soap_header = etree.SubElement(output, '{%s}header' %
                                                                        ns_soap)
                    soap_header.set('use', 'literal')
                    soap_header.set('message', out_header.get_type_name_ns(app))
                    soap_header.set('part', out_header.get_type_name())

                for f in method.faults:
                    fault = etree.SubElement(operation, '{%s}fault' % ns_wsdl)
                    fault.set('name', f.get_type_name())

                    soap_fault = etree.SubElement(fault, '{%s}fault' % ns_soap)
                    soap_fault.set('name', f.get_type_name())
                    soap_fault.set('use', 'literal')

            if method.is_callback:
                relates_to = etree.SubElement(input, '{%s}header' % ns_soap)

                relates_to.set('message', '%s:RelatesToHeader' % pref_tns)
                relates_to.set('part', 'RelatesTo')
                relates_to.set('use', 'literal')

                cb_binding.append(operation)

            else:
                if method.is_async:
                    rt_header = etree.SubElement(input,'{%s}header' % ns_soap)
                    rt_header.set('message', '%s:ReplyToHeader' % pref_tns)
                    rt_header.set('part', 'ReplyTo')
                    rt_header.set('use', 'literal')

                    mid_header = etree.SubElement(input, '{%s}header'% ns_soap)
                    mid_header.set('message', '%s:MessageIDHeader' % pref_tns)
                    mid_header.set('part', 'MessageID')
                    mid_header.set('use', 'literal')

                binding.append(operation)

        return cb_binding



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 13:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0005_auto_20170220_2047'),
        ('soap_1c_api', '0003_auto_20170308_1457'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExtraCollectionRelation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('tag_name', models.CharField(max_length=70, verbose_name='Tag name')),
                ('collection', models.OneToOneField(help_text='Link MetaContent/* tag with a collection.', on_delete=django.db.models.deletion.CASCADE, related_name='extra_collection_relation', to='pcart_catalog.Collection', verbose_name='Collection')),
            ],
            options={
                'verbose_name': 'Extra collection relation',
                'verbose_name_plural': 'Extra collections relations',
            },
        ),
        migrations.AlterField(
            model_name='producttyperelation',
            name='collection',
            field=models.OneToOneField(help_text='Add the products of specified type to a collection.', on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relation', to='pcart_catalog.Collection', verbose_name='Collection'),
        ),
    ]




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""A soap server that uses http as transport, and wsgi as bridge api"""

import logging
logger = logging.getLogger(__name__)

import cgi
import traceback

import soaplib

from soaplib.core.model.exception import Fault

from soaplib.core.mime import apply_mtom
from soaplib.core.mime import collapse_swa
from soaplib.core.util import reconstruct_url
from soaplib.core.server import Base

HTTP_500 = '500 Internal server error'
HTTP_200 = '200 OK'
HTTP_405 = '405 Method Not Allowed'

class ValidationError(Fault):
    pass

def _reconstruct_soap_request(http_env):
    """Reconstruct http payload using information in the http header
    """

    input = http_env.get('wsgi.input')
    length = http_env.get("CONTENT_LENGTH")
    http_payload = input.read(int(length))

    # fyi, here's what the parse_header function returns:
    # >>> import cgi; cgi.parse_header("text/xml; charset=utf-8")
    # ('text/xml', {'charset': 'utf-8'})
    content_type = cgi.parse_header(http_env.get("CONTENT_TYPE"))
    charset = content_type[1].get('charset',None)
    if charset is None:
        charset = 'ascii'

    return collapse_swa(content_type, http_payload), charset

class Application(Base):
    transport = 'http://schemas.xmlsoap.org/soap/http'

    def __call__(self, req_env, start_response, wsgi_url=None):
        '''This method conforms to the WSGI spec for callable wsgi applications
        (PEP 333). It looks in environ['wsgi.input'] for a fully formed soap
        request envelope, will deserialize the request parameters and call the
        method on the object returned by the get_handler() method.

        @param the http environment
        @param a callable that begins the response message
        @param the optional url
        @returns the string representation of the soap call
        '''

        url = wsgi_url
        if url is None:
            url = reconstruct_url(req_env).split('.wsdl')[0]

        if self.__is_wsdl_request(req_env):
            return self.__handle_wsdl_request(req_env, start_response, url)

        elif req_env['REQUEST_METHOD'].lower() != 'post':
            start_response(HTTP_405, [('Allow', 'POST')])
            return ['']

        else:
            return self.__handle_soap_request(req_env, start_response)

    def __is_wsdl_request(self, req_env):
        # Get the wsdl for the service. Assume path_info matches pattern:
        # /stuff/stuff/stuff/serviceName.wsdl or
        # /stuff/stuff/stuff/serviceName/?wsdl

        return (
            req_env['REQUEST_METHOD'].lower() == 'get'
            and (
                   req_env['QUERY_STRING'].endswith('wsdl')
                or req_env['PATH_INFO'].endswith(b'wsdl')
            )
        )

    def __handle_wsdl_request(self, req_env, start_response, url):
        http_resp_headers = {'Content-Type': 'text/xml'}

        try:
            wsdl = self.app.get_wsdl(url)
            self.on_wsdl(req_env, wsdl) # implementation hook

            http_resp_headers['Content-Length'] = str(len(wsdl))
            start_response(HTTP_200, http_resp_headers.items())

            return [wsdl]

        except Exception as e:
            logger.error(traceback.format_exc())

            # implementation hook
            self.on_wsdl_exception(req_env, e)

            start_response(HTTP_500, http_resp_headers.items())

            return [""]

    def __handle_soap_request(self, req_env, start_response):
        ctx = soaplib.core.MethodContext()

        # implementation hook
        self.on_wsgi_call(req_env)

        in_string, in_string_charset = _reconstruct_soap_request(req_env)

        in_object = self.get_in_object(ctx, in_string.decode('utf-8'), in_string_charset)

        return_code = HTTP_200
        if ctx.in_error:
            out_object = ctx.in_error
            return_code = HTTP_500
        else:
            assert ctx.service != None
            out_object = self.get_out_object(ctx, in_object)
            if ctx.out_error:
                out_object = ctx.out_error
                return_code = HTTP_500

        out_string = self.get_out_string(ctx, out_object)

        http_resp_headers = {
            'Content-Type': 'text/xml',
            'Content-Length': '0',
        }

        # implementation hook
        self.on_wsgi_return(req_env, http_resp_headers, out_string)

        if ctx.descriptor and ctx.descriptor.mtom:
            # when there are more than one return type, the result is 
            # encapsulated inside a list. when there's just one, the result
            # is returned unencapsulated. the apply_mtom always expects the
            # objects to be inside an iterable, hence the following test.
            out_type_info = ctx.descriptor.out_message._type_info
            if len(out_type_info) == 1:
                out_object = [out_object]

            http_resp_headers, out_string = apply_mtom(http_resp_headers,
                    out_string, ctx.descriptor.out_message._type_info.values(),
                    out_object)

        # initiate the response
        http_resp_headers['Content-Length'] = str(len(out_string))
        start_response(return_code, http_resp_headers.items())

        return [out_string]

    def on_wsgi_call(self, environ):
        '''This is the first method called when this WSGI app is invoked.

        @param the wsgi environment
        '''
        pass

    def on_wsdl(self, environ, wsdl):
        '''This is called when a wsdl is requested.

        @param the wsgi environment
        @param the wsdl string
        '''
        pass

    def on_wsdl_exception(self, environ, exc):
        '''Called when an exception occurs durring wsdl generation.

        @param the wsgi environment
        @param exc the exception
        @param the fault response string
        '''
        pass

    def on_wsgi_return(self, environ, http_headers, return_str):
        '''Called before the application returns.

        @param the wsgi environment
        @param http response headers as dict
        @param return string of the soap request
        '''
        pass



#!/usr/bin/env python
#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#
import unittest

class FaultTests(unittest.TestCase):

    def _getTargetClass(self):
        from soaplib.core.model.exception import Fault
        return Fault

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_ctor_defaults(self):
        fault = self._makeOne()
        self.assertEqual(fault.faultcode, 'senv:Server')
        self.assertEqual(fault.faultstring, 'Fault')
        self.assertEqual(fault.faultactor, '')
        self.assertEqual(fault.detail, None)
        self.assertEqual(repr(fault), "senv:Server: 'Fault'")

    def test_ctor_faultcode_w_senv_prefix(self):
        fault = self._makeOne(faultcode='senv:Other')
        self.assertEqual(fault.faultcode, 'senv:Other')
        self.assertEqual(repr(fault), "senv:Other: 'Fault'")

    def test_ctor_explicit_faultstring(self):
        fault = self._makeOne(faultstring='Testing')
        self.assertEqual(fault.faultstring, 'Testing')
        self.assertEqual(repr(fault), "senv:Server: 'Testing'")

    def test_ctor_no_faultstring_overridden_get_type_name(self):
        class Derived(self._getTargetClass()):
            def get_type_name(self):
                return 'Overridden'
        fault = Derived()
        self.assertEqual(fault.faultstring, 'Overridden')
        self.assertEqual(repr(fault), "senv:Server: 'Overridden'")

    def test_to_parent_element_wo_detail(self):
        from lxml.etree import Element
        from soaplib.core.namespaces import ns_soap_env
        element = Element('testing')
        fault = self._makeOne()
        cls = self._getTargetClass()

        cls.to_parent_element(fault, 'urn:ignored', element)

        (child,) = element.getchildren()
        self.assertEqual(child.tag, '{%s}Fault' % ns_soap_env)
        self.assertEqual(child.find('faultcode').text, 'senv:Server')
        self.assertEqual(child.find('faultstring').text, 'Fault')
        self.assertEqual(child.find('faultactor').text, '')
        self.failIf(child.findall('detail'))

    def test_to_parent_element_w_detail(self):
        from lxml.etree import Element
        element = Element('testing')
        detail = Element('something')
        fault = self._makeOne(detail=detail)
        cls = self._getTargetClass()

        cls.to_parent_element(fault, 'urn:ignored', element)

        (child,) = element.getchildren()
        self.failUnless(child.find('detail').find('something') is detail)

    def test_add_to_parent_element(self):
        from lxml.etree import Element
        from soaplib.core.namespaces import ns_soap_env
        element = Element('testing')
        fault = self._makeOne()
        cls = self._getTargetClass()

        fault.add_to_parent_element('urn:ignored', element)

        (child,) = element.getchildren()
        self.assertEqual(child.tag, '{%s}Fault' % ns_soap_env)
        self.assertEqual(child.find('faultcode').text, 'senv:Server')
        self.assertEqual(child.find('faultstring').text, 'Fault')
        self.assertEqual(child.find('faultactor').text, '')
        self.failIf(child.findall('detail'))

    def test_from_xml_wo_detail(self):
        from lxml.etree import Element
        from lxml.etree import SubElement
        from soaplib.core.namespaces import ns_soap_env
        element = Element('{%s}Fault' % ns_soap_env)
        fcode = SubElement(element, 'faultcode')
        fcode.text = 'senv:other'
        fstr = SubElement(element, 'faultstring')
        fstr.text = 'Testing'
        actor = SubElement(element, 'faultactor')
        actor.text = 'phreddy'

        fault = self._getTargetClass().from_xml(element)

        self.assertEqual(fault.faultcode, 'senv:other')
        self.assertEqual(fault.faultstring, 'Testing')
        self.assertEqual(fault.faultactor, 'phreddy')
        self.assertEqual(fault.detail, None)

    def test_from_xml_w_detail(self):
        from lxml.etree import Element
        from lxml.etree import SubElement
        from soaplib.core.namespaces import ns_soap_env
        element = Element('{%s}Fault' % ns_soap_env)
        fcode = SubElement(element, 'faultcode')
        fcode.text = 'senv:other'
        fstr = SubElement(element, 'faultstring')
        fstr.text = 'Testing'
        actor = SubElement(element, 'faultactor')
        actor.text = 'phreddy'
        detail = SubElement(element, 'detail')

        fault = self._getTargetClass().from_xml(element)

        self.failUnless(fault.detail is detail)

    def test_add_to_schema_no_extends(self):
        from soaplib.core.namespaces import ns_xsd
        class cls(self._getTargetClass()):
            @classmethod
            def get_type_name_ns(self, app):
                return 'testing:My'

        schema = DummySchemaEntries(object())

        cls.add_to_schema(schema)

        self.assertEqual(len(schema._complex_types), 1)
        c_cls, c_elt = schema._complex_types[0]
        self.failUnless(c_cls is cls)
        self.assertEqual(c_elt.tag, '{%s}complexType' % ns_xsd)
        self.assertEqual(c_elt.get('name'), 'FaultFault')
        self.assertEqual(len(c_elt), 1)
        seq = c_elt[0]
        self.assertEqual(seq.tag, '{%s}sequence' % ns_xsd)

        self.assertEqual(len(schema._elements), 1)
        e_cls, e_elt = schema._elements[0]
        self.failUnless(e_cls is cls)
        self.assertEqual(e_elt.tag, '{%s}element' % ns_xsd)
        self.assertEqual(e_elt.get('name'), 'Fault')
        self.assertEqual(e_elt.get('{%s}type' % ns_xsd), 'testing:MyFault')
        self.assertEqual(len(e_elt), 0)

    def test_add_to_schema_w_extends(self):
        from soaplib.core.namespaces import ns_xsd
        class base(self._getTargetClass()):
            @classmethod
            def get_type_name_ns(self, app):
                return 'testing:Base'
        class cls(self._getTargetClass()):
            __extends__ = base
            @classmethod
            def get_type_name_ns(self, app):
                return 'testing:My'

        schema = DummySchemaEntries(object())

        cls.add_to_schema(schema)

        self.assertEqual(len(schema._complex_types), 1)
        c_cls, c_elt = schema._complex_types[0]
        self.failUnless(c_cls is cls)
        self.assertEqual(c_elt.tag, '{%s}complexType' % ns_xsd)
        self.assertEqual(c_elt.get('name'), 'FaultFault')
        self.assertEqual(len(c_elt), 1)

        cc_elt = c_elt[0]
        self.assertEqual(cc_elt.tag, '{%s}complexContent' % ns_xsd)
        self.assertEqual(len(cc_elt), 1)

        e_elt = cc_elt[0]
        self.assertEqual(e_elt.tag, '{%s}extension' % ns_xsd)
        self.assertEqual(e_elt.get('base'), 'testing:Base')
        self.assertEqual(len(e_elt), 1)

        seq = e_elt[0]
        self.assertEqual(seq.tag, '{%s}sequence' % ns_xsd)

        self.assertEqual(len(schema._elements), 1)
        e_cls, e_elt = schema._elements[0]
        self.failUnless(e_cls is cls)
        self.assertEqual(e_elt.tag, '{%s}element' % ns_xsd)
        self.assertEqual(e_elt.get('name'), 'Fault')
        self.assertEqual(e_elt.get('{%s}type' % ns_xsd), 'testing:MyFault')
        self.assertEqual(len(e_elt), 0)

class DummySchemaEntries:

    def __init__(self, app):
        self.app = app
        self._complex_types = []
        self._elements = []

    def add_complex_type(self, cls, ct):
        self._complex_types.append((cls, ct))

    def add_element(self, cls, elt):
        self._elements.append((cls, elt))


if __name__ == '__main__': #pragma NO COVERAGE
    unittest.main()




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""A soap server that uses http as transport, and wsgi as bridge api"""

from lxml import etree

import logging
logger = logging.getLogger(__name__)

from soaplib.core.model.exception import Fault
from soaplib.core.model.primitive import string_encoding

HTTP_500 = '500 Internal server error'
HTTP_200 = '200 OK'
HTTP_405 = '405 Method Not Allowed'

class ValidationError(Fault):
    pass

class Base(object):
    transport = None

    def __init__(self, app):
        self.app = app
        self.app.transport = self.transport

    def get_in_object(self, ctx, in_string, in_string_charset=None):
        in_object = None
        root, xmlids = self.app.parse_xml_string(in_string, in_string_charset)

        try:
            in_object = self.app.deserialize_soap(ctx, self.app.IN_WRAPPER,
                                                                   root, xmlids)
        except Fault as e:
            ctx.in_error = e

        return in_object

    def get_out_object(self, ctx, in_object):
        out_object = self.app.process_request(ctx, in_object)

        if isinstance(out_object, Fault):
            ctx.out_error = out_object
        else:
            assert not isinstance(out_object, Exception)

        return out_object

    def get_out_string(self, ctx, out_object):
        out_xml = self.app.serialize_soap(ctx, self.app.OUT_WRAPPER, out_object)
        out_string = etree.tostring(out_xml, xml_declaration=True,
                                                       encoding=string_encoding)
        return out_string



from ftplib import FTP
from io import BytesIO
from lxml import etree, objectify
from lxml.html import tostring
from urllib.request import Request, urlopen
from django.core.files.storage import default_storage

import pytils
import os
import re
import logging
import traceback

from django.conf import settings
from django.core.files import File
from django.core.files.base import ContentFile
from django.contrib.sites.models import Site
from pcart_catalog.models import (
    Collection,
    ProductType,
    ProductTypeProperty,
    Vendor,
    Product,
    ProductVariant,
    ProductImage,
    ProductStatus)
from .models import (
    XMLFile,
    ProductImportLogMessage,
    ProductTypeRelation,
    ExtraCollectionRelation,
)


logger = logging.getLogger('import1c')


DEFAULT_STATUS_TITLE = 'Нет в наличии'


def import_goods_from_xml(sender, path, task_instance=None):
    """Load a catalog from the XML file
    """
    xml = etree.parse(default_storage.open(path, 'rb'))  # get etree of file

    categories = xml.xpath('Classifier/Category')
    categories_errors = parse_categories(categories, task_instance=task_instance)

    if categories_errors:
        for k, v in categories_errors.items():
            logger.error('%s : %s' % (k, v))

    products = xml.xpath('Catalog/Product')
    products_errors = parse_products(products, task_instance=task_instance)

    if products_errors:
        for k, v in products_errors.items():
            logger.error('%s : %s' % (k, v))
    return products_errors


def get_first_or_empty(obj):
    try:
        return obj[0].text
    except IndexError:
        return ''


def parse_categories(categories, task_instance=None):
    """ Load product types and colelctions from the Classifier/Category
    section from the XML file.
    """
    TOP_CATEGORY = '00000000-0000-0000-0000-000000000000'
    site = Site.objects.get_current()

    errors = {}
    for category_tag in categories:
        uuid = category_tag.get('UUID')
        try:
            published = True

            marked_for_delete = category_tag.get('Marked')
            parent = category_tag.get('Parent')
            title = category_tag.get('Name')

            # Unpublish marked collections
            if marked_for_delete == 'true':
                published = False

            try:
                pt_relation = ProductTypeRelation.objects.get(
                    alternate_uuid=uuid)

                save_collection = False
                save_product_type = False

                _product_type = pt_relation.product_type
                _collection = pt_relation.collection

                if not published and _collection.published is True:
                    _collection.published = published  # Unpublish
                    save_collection = True
                if title != _collection.title:
                    _collection.title = title  # Change Collection title
                    save_collection = True
                if title != _product_type.title:
                    _product_type.title = title  # Change ProductType title
                    save_product_type = True
                if parent == TOP_CATEGORY and _collection.parent is not None:
                    _collection.parent = None  # Move a collection to the top
                    save_collection = True
                if parent != TOP_CATEGORY:
                    try:
                        parent_pt_relation = ProductTypeRelation.objects.get(
                            alternate_uuid=parent)
                        # Change a collection's parent
                        _parent_collection = parent_pt_relation.collection
                        if _collection.parent != _parent_collection:
                            _collection.parent = _parent_collection
                            save_collection = True
                    except ProductTypeRelation.DoesNotExist:
                        errors[uuid] = 'Unknown parent category %s' % parent

                if title != pt_relation.title:
                    pt_relation.title = title
                    pt_relation.save()

                if save_collection:
                    _collection.save()
                if save_product_type:
                    _product_type.save()

            except ProductTypeRelation.DoesNotExist:
                try:
                    product_type = ProductType.objects.get(title=title)
                except ProductType.DoesNotExist:
                    product_type = ProductType.objects.create(title=title)

                try:
                    collection = Collection.objects.get(title=title)
                except Collection.DoesNotExist:
                    collection = Collection(
                        site=site,
                        title=title,
                        slug=get_unique_collection_slug(title),
                        published=published,
                    )
                    if parent != TOP_CATEGORY:
                        try:
                            parent_pt_relation = \
                                ProductTypeRelation.objects.get(
                                    alternate_uuid=parent)
                            # Change a collection's parent
                            collection.parent = parent_pt_relation.collection
                        except ProductTypeRelation.DoesNotExist:
                            errors[uuid] = \
                                'Unknown parent category %s' % parent
                    collection.save()

                pt_relation = ProductTypeRelation(
                    title=title,
                    alternate_uuid=uuid,
                    product_type=product_type,
                    collection=collection,
                )
                pt_relation.save()
        except Exception as e:
            errors[uuid] = '%s' % str(e)

    return errors


def parse_products(products, task_instance=None):
    """ Load products from the Catalog/Product section from the XML file.
    """
    errors = {}

    extra_collection_relations = ExtraCollectionRelation.objects.all()

    if task_instance:
        size = len(products)
        counter = 0

    for product_tag in products:
        if task_instance:
            counter += 1
            task_instance.update_state(
                state='PROGRESS', meta={'progress': int(counter/size*100)})

        uuid = product_tag.get('UUID')
        try:
            published = True
            category = product_tag.get('Parent')

            id_1c = product_tag.get('ID1C').strip()
            marked_for_delete = product_tag.get('Marked')
            title = get_first_or_empty(product_tag.xpath('Name')).strip()
            prices = product_tag.xpath('Prices/Price')

            # Unpublish marked products
            if marked_for_delete == 'true':
                published = False

            # SKU
            try:
                sku = get_first_or_empty(product_tag.xpath('Article'))
            except IndexError:
                sku = ''

            # Vendor
            try:
                vendor = get_first_or_empty(product_tag.xpath('Manufacturer'))
            except IndexError:
                vendor = ''

            # Short description
            # try:
            #     short_description = product_tag.xpath(
            #         u'MetaContent/DescriptionShort')[0].text
            # except IndexError:
            #     short_description = u''

            # Description
            try:
                description = get_first_or_empty(product_tag.xpath(
                    'MetaContent/DescriptionFull'))
            except IndexError:
                description = ''

            # Status
            try:
                status = get_first_or_empty(product_tag.xpath('Status'))
            except IndexError:
                status = DEFAULT_STATUS_TITLE

            # product with variant
            try:
                parent_product = get_first_or_empty(product_tag.xpath(
                    'ParentProduct'))
            except IndexError:
                parent_product = ''

            # sale price of product
            try:
                compare_at_price = product_tag.xpath('Prices/SalePrice')
            except IndexError:
                compare_at_price = None

            # Warranty
            try:
                warranty = get_first_or_empty(product_tag.xpath('Warranty'))
                warranty = int(warranty)
            except:
                warranty = 0

            # IsParent
            try:
                is_parent = get_first_or_empty(product_tag.xpath('IsParent'))
                is_parent = True if is_parent.lower() == 'true' else False
            except:
                is_parent = False

            # Find the item type (product or variant)
            item_type = 'product'
            if is_parent is False and parent_product != '':
                item_type = 'variant'

            # TODO: work with unit
            # unit = product_tag.xpath('/Unit')[0]
            # unit_id = unit.get(u'ID')
            # unit_name = unit.get(u'Name')
            # unit_full_name = unit.get(u'FullName')
            # unit_inter = unit.get(u'International')

            # TODO: check if to use this
            # quantity = product_tag.xpath('Quantity')[0].text

            pictures = product_tag.xpath('Pictures/Picture')

            # Options
            options = product_tag.xpath('Options/Option')

            # Special meta content
            meta_content = product_tag.xpath('MetaContent')

            # Default values
            page_title = meta_description = ''
            extra_collections = []

            if meta_content:
                meta_content = meta_content[0]
                page_title = \
                    get_first_or_empty(meta_content.xpath('METATitle'))
                meta_description = get_first_or_empty(
                    meta_content.xpath('METADescription'))
                # meta_keywords = meta_content.xpath('METAKeywords')

                for ecr in extra_collection_relations:
                    # Check for: Accesories, New, Popular and Recommended
                    t = get_first_or_empty(meta_content.xpath(ecr.tag_name))
                    if t and t.lower() == 'true':
                        extra_collections.append(ecr.collection)

            # print('::', uuid, item_type, is_parent, parent_product)

            if item_type == 'product':
                obj, created = get_or_create_product(
                    id=uuid,
                    title=title,
                    description=description,
                    page_title=page_title,
                    meta_description=meta_description,
                    vendor=vendor,
                    category=category,
                    sku=sku,
                    barcode=id_1c,
                    price=get_product_price(prices),
                    compare_at_price=compare_at_price,
                    published=published,
                    status=status,
                    options=options,
                    extra_collections=extra_collections,
                )
            else:
                obj, created = get_or_create_variant(
                    id=uuid,
                    title=title,
                    parent_product=parent_product,
                    category=category,
                    sku=sku,
                    barcode=id_1c,
                    status=status,
                    price=get_product_price(prices),
                    compare_at_price=compare_at_price,
                    options=options,
                )

            # if product:
            #     ProductImportLog.objects.create(
            #         product=product,
            #         file_link=file_link,
            #         job=job,
            #         text=tostring(product_tag),
            #     )

            # # accessories
            # if created or UPDATE_1C_ACCESSORIES:
            #     try:
            #         accessories = product_tag.xpath(
            #             'Accesories/Accessory')  # TODO: fix "Accesories"
            #         uids = [a.text for a in accessories]
            #         accessories = Product.objects.filter(uid__in=uids)
            #         for acc in accessories:
            #             ProductAccessories.objects.get_or_create(
            #                 product=product, accessory=acc)
            #     except:
            #         pass

            # # recommended products for product
            # if UPDATE_1C_RELATED:
            #     recommendeds = product_tag.xpath(
            #         u'RecommendedProducts/RecommendedProduct')
            #     product.related_products.clear()
            #     for recommended in recommendeds:
            #         r = Product.objects.get(uid=recommended.text)
            #         product.related_products.add(r)

            # Images
            # # from tasks.models import Job

            # pictures_list = []
            # for p in pictures:
            #     pictures_list.append({
            #         'URL': p.get('URL'),
            #         'Name': p.get('Name'),
            #         'Marked': p.get('Marked'),
            #         'Default': p.get('Default'),
            #     })
            # products_images_dict[product.uid] = pictures_list

            # TODO: should think about how to be with images for variants
            if item_type == 'product':
                img_target_obj = obj
            else:
                img_target_obj = obj.product

            update_product_images(img_target_obj, pictures)
            # load_images(product, pictures)

            # # Options
            # if created or UPDATE_1C_OPTIONS:
            #     load_options(product, options, parent)

            # uuids.append(uuid)
            # if 'tetradka' in settings.INSTALLED_APPS:
            #     from tetradka.listeners import product_saved_listener
            #     product_saved_listener(None, product)
        except Exception as e:
            raise e
            # if getattr(settings, 'WRITE_ERRORS_TO_LOG', False):
            logger.error(str(e))
            errors[uuid] = '%s' % str(e)
    # job = Job.objects.create(
    #     name='soap_1c_api.jobs.product_load_images',
    #     duty=1,
    #     status='success'
    # )
    # job.kwargs = products_images_dict
    # job.save()
    # if uids:
    #     mediator.publish(None, 'similar_products:make', uids)
    return errors


def update_product_images(product, pictures):
    pictures_list = []
    for p in pictures:
        pictures_list.append({
            'URL': p.get('URL'),
            'Name': p.get('Name'),
            'Marked': p.get('Marked'),
            'Default': p.get('Default'),
        })

    p_images = product.images.all()
    old_pictures_list = []
    old_links = []
    for im in p_images:
        # Ignore images or snippets which could be added manually
        if im.download_link:
            old_links.append(im.download_link)
            old_pictures_list.append({
                'URL': im.download_link,
                'Name': im.title,
            })

    for p in pictures_list:
        if p['URL'] in old_links:
            if p['Marked'] == 'true':
                # Delete marked image
                product.images.filter(download_link=p['URL']).delete()
                logger.warning(
                    '%s : Image %s has been removed.' % (product.id, p['URL']))
            else:
                p_ims = product.images.filter(download_link=p['URL'])
                p_ims.update(title=p['Name'])
        else:
            # Add new image
            image = ProductImage(
                title=p['Name'] or product.title,
                product=product,
                download_link=p['URL'],
            )
            image.save()
            # Now the signal should raised after saving and
            # the new task for download the file automatically
            # have be started via Celery. Do not worry about that here.


def get_or_create_status(status):
    try:
        result = ProductStatus.objects.get(title=status)
    except:
        result = ProductStatus.objects.create(
            title=status,
            show_buy_button=True,
            is_visible=True,
            is_searchable=True
        )
    return result


def get_or_create_vendor(vendor):
    try:
        result = Vendor.objects.get(title=vendor)
    except Vendor.DoesNotExist:
        result = Vendor.objects.create(title=vendor)
    return result


def get_product_type_and_collection(category):
    try:
        pt_relation = ProductTypeRelation.objects.get(alternate_uuid=category)
        product_type = pt_relation.product_type
        collection = pt_relation.collection
    except ProductTypeRelation.DoesNotExist:
        return None, None
    return product_type, collection


def get_unique_collection_slug(title):
    import itertools
    _slug = _orig = pytils.translit.slugify(title)
    for x in itertools.count(1):
        if not Collection.objects.filter(slug=_slug).exists():
            break
        _slug = '%s-%d' % (_orig, x)
    return _slug


def get_unique_product_slug(title):
    import itertools
    _slug = _orig = pytils.translit.slugify(title)
    for x in itertools.count(1):
        if not Product.objects.filter(slug=_slug).exists():
            break
        _slug = '%s-%d' % (_orig, x)
    return _slug


def get_unique_variant_slug(product_id, title):
    import itertools
    _slug = _orig = pytils.translit.slugify(title)
    for x in itertools.count(1):
        if not ProductVariant.objects.filter(
                product_id=product_id, slug=_slug).exists():
            break
        _slug = '%s-%d' % (_orig, x)
    return _slug


def get_or_create_product(
        id, title, description, page_title, meta_description,
        vendor, category, sku, barcode,
        price, compare_at_price,
        published, status, options,
        extra_collections=[]):
    """ Create or update parent product.
    """

    status_obj = get_or_create_status(status)
    vendor_obj = get_or_create_vendor(vendor)
    product_type, collection = get_product_type_and_collection(category)
    update_product_type_properties(options, category)

    params = {
        'title': title,
        'sku': sku,
        'barcode': barcode,
        'status': status_obj,
        'vendor': vendor_obj,
        'price': price or 0.0,
        'compare_at_price': compare_at_price or 0.0,
        'published': published,
        'properties': get_properties_dict(options),
        'product_type': product_type,
    }

    if getattr(settings, 'UPDATE_1C_SEO_PRODUCTS', True):
        params.update({
            'page_title': page_title,
            'meta_description': meta_description,
        })

    if getattr(settings, 'UPDATE_1C_PRODUCTS_DESCRIPTION', True):
        params.update({
            'description': description,
        })

    create = False
    try:
        product = Product.objects.get(id=id)
        for k, v in params.items():
            setattr(product, k, v)
        product.save()
        if not product.collections.filter(id=collection.id).exists():
            product.collections.add(collection)
    except Product.DoesNotExist:
        params.update({
            'id': id,
            'slug': get_unique_product_slug(title),
        })
        product = Product(**params)
        product.save()
        create = True
        product.collections.add(collection)

    if extra_collections:
        product.collections.add(*extra_collections)

    return product, create


def get_or_create_variant(
        id, title, parent_product,
        category,
        sku, barcode, status,
        price, compare_at_price,
        options):
    """ Create or update variant product.
    """
    status_obj = get_or_create_status(status)
    update_product_type_properties(options, category)

    params = {
        'title': title,
        'sku': sku,
        'barcode': barcode,
        'status': status_obj,
        'product_id': parent_product,
        'price': price or 0.0,
        'compare_ar_price': compare_at_price or 0.0,
        # 'published': published,
        'properties': get_properties_dict(options),
    }
    create = False
    try:
        variant = ProductVariant.objects.get(id=id)
        for k, v in params.items():
            setattr(variant, k, v)
        variant.save()
    except ProductVariant.DoesNotExist:
        params.update({
            'id': id,
            'product_id': parent_product,
            'slug': get_unique_variant_slug(parent_product, title),
        })
        variant = ProductVariant(**params)
        variant.save()
        create = True

    return variant, create


def update_product_type_properties(options, alternate_uuid):
    try:
        pt_relation = ProductTypeRelation.objects.get(
            alternate_uuid=alternate_uuid)
        product_type = pt_relation.product_type

        _properties = [option.get('Name') for option in options]
        for p in _properties:
            try:
                ProductTypeProperty.objects.get(
                    title=p, product_type=product_type)
            except ProductTypeProperty.DoesNotExist:
                ProductTypeProperty.objects.create(
                    title=p, product_type=product_type)
    except ProductTypeRelation.DoesNotExist:
        logger.warning(
            'Product type relation for %s does not exist.' % alternate_uuid)


def get_properties_dict(options):
    _properties = dict()
    for option in options:
        # group_name = option.get('Group')  # do not supported yet
        property_name = option.get('Name')
        property_value = option.get('Value')
        _properties[property_name] = property_value
    return _properties


def get_product_price(prices):
    for price in prices:
        price_for_unit = get_first_or_empty(price.xpath('PriceForUnit'))
        price = float(price_for_unit.replace(',', '.'))
        return price
    return 0.0




from django.test.testcases import TestCase
from soap-server.wsgi import embed_wsgi

def simple_wsgi(*args, **kwargs):
    """ A simple webservice for test purposes"""
    return ['']

class SoapLibHandlerTestCase(TestCase):
    
    def test_unsecure_call_returns_a_400(self):
        request = Mock()
        setattr(request, 'is_secure', lambda:False)
        app = embed_wsgi(simple_wsgi)
        res = app(request)
        self.assertEqual(res.status_code, 400)
    
    def test_secure_unauthenticated_call_returns_a_401(self):
        request = Mock()
        setattr(request, 'is_secure', lambda:True)
        setattr(request, 'environ', {})
        app = embed_wsgi(simple_wsgi)
        res = app(request)
        self.assertEqual(res.status_code, 401)
    
    def test_secure_authenticated_call_returns_a_405(self):
        request = Mock()
        setattr(request, 'is_secure', lambda:True)
        setattr(request, 'environ', {})
        app = embed_wsgi(simple_wsgi)
        res = app(request)
        self.assertEqual(res.status_code, 405)
        
    # TODO Make a 200 check




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from soaplib.core import namespaces
from lxml import etree
from soaplib.core.model import Base

_ns_xsd = namespaces.ns_xsd
_pref_soap_env = namespaces.const_prefmap[namespaces.ns_soap_env]

class Fault(Exception, Base):
    __type_name__ = "Fault"

    def __init__(self, faultcode='Server', faultstring="",
                 faultactor="", detail=None):
        if faultcode.startswith('%s:' % _pref_soap_env):
            self.faultcode = faultcode
        else:
            self.faultcode = '%s:%s' % (_pref_soap_env, faultcode)

        self.faultstring = faultstring or self.get_type_name()
        self.faultactor = faultactor
        self.detail = detail

    def __repr__(self):
        return "%s: %r" % (self.faultcode, self.faultstring)

    @classmethod
    def to_parent_element(cls, value, tns, parent_elt, name=None):
        assert name is None
        element = etree.SubElement(parent_elt,
                                   "{%s}Fault" % namespaces.ns_soap_env)

        etree.SubElement(element, 'faultcode').text = value.faultcode
        etree.SubElement(element, 'faultstring').text = value.faultstring
        etree.SubElement(element, 'faultactor').text = value.faultactor
        if value.detail != None:
            etree.SubElement(element, 'detail').append(value.detail)

    def add_to_parent_element(self, tns, parent):
        self.__class__.to_parent_element(self, tns, parent, name=None)

    @classmethod
    def from_xml(cls, element):
        code = element.find('faultcode').text
        string = element.find('faultstring').text
        factor = element.find('faultactor').text
        detail = element.find('detail')

        return cls(faultcode=code, faultstring=string,
                   faultactor=factor, detail=detail)

    @classmethod
    def add_to_schema(cls, schema_dict):
        app = schema_dict.app
        complex_type = etree.Element('{%s}complexType' % _ns_xsd)
        complex_type.set('name', '%sFault' % cls.get_type_name())

        extends = getattr(cls, '__extends__', None)
        if extends is not None:
            complex_content = etree.SubElement(complex_type,
                                            '{%s}complexContent' % _ns_xsd)
            extension = etree.SubElement(complex_content, "{%s}extension"
                                                            % namespaces.ns_xsd)
            extension.set('base', extends.get_type_name_ns(app))
            sequence_parent = extension
        else:
            sequence_parent = complex_type

        seq = etree.SubElement(sequence_parent, '{%s}sequence' % _ns_xsd)

        schema_dict.add_complex_type(cls, complex_type)

        top_level_element = etree.Element('{%s}element' % _ns_xsd)
        top_level_element.set('name', cls.get_type_name())
        top_level_element.set('{%s}type' % _ns_xsd,
                              '%sFault' % cls.get_type_name_ns(app))

        schema_dict.add_element(cls, top_level_element)



#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

# namespace map

ns_xml = 'http://www.w3.org/XML/1998/namespace'
ns_xsd = 'http://www.w3.org/2001/XMLSchema'
ns_xsi = 'http://www.w3.org/2001/XMLSchema-instance'
ns_plink = 'http://schemas.xmlsoap.org/ws/2003/05/partner-link/'
ns_soap = 'http://schemas.xmlsoap.org/wsdl/soap/'
ns_wsdl = 'http://schemas.xmlsoap.org/wsdl/'
ns_soap_enc = 'http://schemas.xmlsoap.org/soap/encoding/'
ns_soap_env = 'http://schemas.xmlsoap.org/soap/envelope/'
ns_soap12_env = 'http://www.w3.org/2003/05/soap-envelope/'
ns_soap12_enc = 'http://www.w3.org/2003/05/soap-encoding/'
ns_wsa = 'http://schemas.xmlsoap.org/ws/2003/03/addressing'
ns_xop = 'http://www.w3.org/2004/08/xop/include'

const_nsmap = {
    'xml': ns_xml,
    'xs': ns_xsd,
    'xsi': ns_xsi,
    'plink': ns_plink,
    'soap': ns_soap,
    'wsdl': ns_wsdl,
    'senc': ns_soap_enc,
    'senv': ns_soap_env,
    's12env': ns_soap12_env,
    's12enc': ns_soap12_enc,
    'wsa': ns_wsa,
    'xop': ns_xop,
}

const_prefmap = dict([(b,a) for a,b in const_nsmap.items()])



from soaplib.core.service import DefinitionBase, soap
from soaplib.core.model.primitive import String  # , Integer, Double
from soap_1c_api.models import XMLFile
from lxml import etree
from io import BytesIO
from django.conf import settings
from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from soap_1c_api.py2xml import Py2XML

import time
import hashlib
import zipfile
import base64


# def run_1c_task(path, job_meta={}):
#     job_id = job_meta['id']
#     t = mediator.publish(None, '1c_exchange:import', path, job_id)
#     result = get_message_result('1c_exchange:import', t)
#     return {'1c_exchange:import': result}


def write_to_file(xml):
    path = default_storage.save(
        '1c_exchange/%s.xml' % hashlib.sha1(
            str(time.ctime()).encode('utf-8')).hexdigest(),
        ContentFile(xml))
    return path


def get_as_xml(obj):
    items = ''
    for data in obj.get_data():
        attrs = u''
        for key in data:
            attrs += u' {key}="{value}"'.format(key=key, value=data[key])
        items += u'<{item_name}{attrs} />\n'.format(
            item_name=obj.item_name, attrs=attrs
        )
    return u'<{name}>\n{items}</{name}>'.format(name=obj.name, items=items)


class ExchangeWebService(DefinitionBase):
    '''SOAP Import/Export interface for 1C
    '''
    def __init__(self, environ):
        '''
        This saves a reference to the request environment
        on the current instance
        '''
        self.environ = environ
        super(ExchangeWebService, self).__init__(environ)

    @soap(String, _returns=String)
    def import_goods(self, xml_body):
        xml_body = base64.b64decode(xml_body)
        try:
            zip_xml = zipfile.ZipFile(BytesIO(xml_body))
            xml_body = zip_xml.open(zip_xml.namelist()[0]).read()
        except zipfile.BadZipfile:
            pass  # TODO: add logger info
        '''Require the next xml:
        <timeout>10</timeout>
        '''
        # scs = 10
        scs = len(xml_body) / 1000
        path = write_to_file(xml_body)
        # create xmlfile in DB
        t = XMLFile.objects.create(filename=path)
        r = t.run_import_task()
        return base64.b64encode(bytes('''<result>
<task_id>%s</task_id>
<waiting_time>%d</waiting_time>
<success>True</success>
</result>''' % (r.id, scs + 30), 'utf8'))

    @soap(_returns=String)
    def export_orders(self):
        from pcart_cart import Order
        from collections import OrderedDict
        from xml.sax.saxutils import escape
        import pytz

        ESCAPE_CHARS = {
            '"': "&quot;",
            "'": "&apos;",
            "&": "&amp;",
            ">": "&gt;",
            "<": "&lt;",
        }

        def text_escape(text):
            return escape(text, ESCAPE_CHARS)

        message = ''

        orders = []
        queryset = Order.objects.filter(export_status='notset')
        for order in queryset:
            products_items = []
            services = []
            for i in order.items():
                if i['item_type'] in ['product', 'variant']:
                    products_items.append(OrderedDict([
                        ('product_name', text_escape(i['object'].title)),
                        ('product_price', i['price']),
                        ('product_amount', i['quantity']),
                        ('price', i['line_price']),
                        ('product__uid', i['item_id']),
                        ('product__price', i['object'].price),
                    ]))
                else:
                    services.append(OrderedDict([
                        ('product_name', text_escape(i['object'].title)),
                        ('product_price', i['price']),
                    ]))

            orders.append(OrderedDict([
                ('id', order.id),
                ('datetime', '{:%Y-%m-%d%H%M%S}'.format(
                    order.added.astimezone(
                        pytz.timezone(settings.TIME_ZONE)
                    )
                )),
                ('number', text_escape(order.number)),
                ('price', order.total_price),
                ('currency', settings.PCART_DEFAULT_CURRENCY),
                ('customer', {
                    'id': order.customer.id,
                    'full_name': order.customer.name,
                    'phones': [{
                        'number': order.customer.user.phone,
                        'type': '-----',
                        }],
                    'addresses': [{
                        'value': text_escape(order.get_shipping_info()),
                        }],
                    'email': {
                        'value': order.customer.user.email,
                    },
                }),
                ('shippingmethod', {
                    'type': text_escape(order.get_shipping_method_name()),
                }),
                ('paymentmethod', {
                    'type': text_escape(order.get_payment_method_name()),
                }),
                ('products', products_items),
                ('services', services),
                ('comment', order.note),
            ]))

        message += Py2XML().parse(
            {'orders': orders}).replace('><', '>\n<')
        if not message:
            message = '''<result>
<task_id>0</task_id>
<waiting_time>0</waiting_time>
<success>True</success>
</result>'''
        message = '<v8msg>%s</v8msg>' % message
        return base64.b64encode(bytes(message, 'utf8'))

    @soap(String, _returns=String)
    def set_export_status(self, xml_body):
        from pcart_cart.models import Order, OrderStatus
        xml_body = base64.b64decode(xml_body)
        zip_xml = zipfile.ZipFile(BytesIO(xml_body))
        xml_body = zip_xml.open(zip_xml.namelist()[0]).read()
        root = etree.XML(xml_body)
        received = root.xpath('orders/received/order')
        statuses = root.xpath('orders/statuses')

        if received:
            received_numbers = [a.get('number') for a in received]
            queryset = Order.objects.filter(number__in=received_numbers)
            queryset.update(export_status='success')

        if statuses:
            data = statuses[0].getchildren()
            for chunk in data:
                if 'status' in chunk and 'number' in chunk:
                    try:
                        status = OrderStatus.objects.get(
                            slug=chunk['status'])
                    except OrderStatus.DoesNotExist:
                        status = OrderStatus(
                            name=chunk['status'].capitalize(),
                            slug=chunk['status'],
                        )
                        status.save()
                    Order.objects.filter(number=chunk['number'])\
                        .update(status=status)

        return base64.b64encode(bytes('''<result>
<success>True</success>
</result>''', 'utf8'))

    @soap(String, _returns=String)
    def check_status(self, task_id):
        from .tasks import import_goods
        task = import_goods.AsyncResult(task_id)
        status = task.status

        if status == 'PROGRESS':
            status = 'pending'
        elif status == 'SUCCESS':
            status = 'success'
        elif status == 'FAILURE':
            status = 'failed'
        return base64.b64encode(bytes('''<result>
<status>%s</status>
</result>''' % status, 'utf8'))

    @soap(String, _returns=String)
    def get_import_result(self, task_id):
        from .tasks import import_goods
        task = import_goods.AsyncResult(task_id)
        result = task.result
        output = ''
        if result:
            for key, value in result.items():
                output += '<error uuid="%s" message="%s" />\n' % (key, value)
        return base64.b64encode(bytes('''<v8msg>
<errors>%s</errors>
</v8msg>''' % output, 'utf8'))

    @soap(_returns=String)
    def pending_tasks(self):
        # pending_tasks = task_manager.get_pending_tasks()
        pending_tasks = []
        output = '\n'.join(
            ['<task_id>%d</task_id>' % x for x in pending_tasks]
        )
        return base64.b64encode(bytes('''<result>
%s
</result>''' % output, 'utf8'))




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""
Module for soaplib model base classes (Base, SimpleType, and Null),
along with several helper functions.
"""


from soaplib.core import namespaces

from lxml import etree

__all__ = ('nillable_value','nillable_element','nillable_string','Base','Null',
           'SimpleType')

def nillable_value(func):
    def wrapper(cls, value, tns, parent_elt, *args, **kwargs):
        if value is None:
            Null.to_parent_element(value, tns, parent_elt, *args, **kwargs)
        else:
            func(cls, value, tns, parent_elt, *args, **kwargs)
    return wrapper

def nillable_element(func):
    def wrapper(cls, element):
        if bool(element.get('{%s}nil' % namespaces.ns_xsi)):
            return None
        else:
            return func(cls, element)
    return wrapper

def nillable_string(func):
    def wrapper(cls, string):
        if string is None:
            return None
        else:
            return func(cls, string)
    return wrapper

class Base(object):
    """
    Base class for all soaplib models.
    """
    __namespace__ = None
    __type_name__ = None

    # There are not the xml schema defaults. The xml schema defaults are
    # considered in ClassModel's add_to_schema method. the defaults here
    # are to reflect what people want most.
    class Attributes(object):
        nillable = True
        min_occurs = 0
        max_occurs = 1

    class Annotations(object):
        doc = ""

    class Empty(object):
        pass

    @staticmethod
    def is_default(cls):
        return True

    @classmethod
    def get_namespace_prefix(cls,app):
        ns = cls.get_namespace()

        retval = app.get_namespace_prefix(ns)

        return retval

    @classmethod
    def get_namespace(cls):
        return cls.__namespace__

    @staticmethod
    def resolve_namespace(cls, default_ns):
        if cls.__namespace__ in namespaces.const_prefmap and not cls.is_default(cls):
            cls.__namespace__ = default_ns

        if cls.__namespace__ is None:
            cls.__namespace__ = cls.__module__

    @classmethod
    def get_type_name(cls):
        retval = cls.__type_name__
        if retval is None:
            retval = cls.__name__.lower()

        return retval

    @classmethod
    def get_type_name_ns(cls,app):
        if cls.get_namespace() != None:
            return "%s:%s" % (cls.get_namespace_prefix(app),cls.get_type_name())

    @classmethod
    @nillable_element
    def from_xml(cls, element):
        return cls.from_string(element.text)

    @classmethod
    @nillable_value
    def to_parent_element(cls, value, tns, parent_elt, name='retval'):
        '''
        Creates a lxml.etree SubElement as a child of a 'parent' Element
        @param The value to be set for the 'text' element of the newly created
        SubElement
        @param The target namespace of the new SubElement, used with 'name' to
        set the tag.  
        @param The parent Element to which the new child will be appended.
        @param The new tag name of new SubElement.
        '''

        assert isinstance(value, str), \
            "'value' must be string or unicode. it is instead %r" % value

        etree.SubElement(parent_elt, "{%s}%s" % (tns,name)).text = value

    @classmethod
    def add_to_schema(cls, schema_entries):
        '''
        Add this type to the wsdl.
        '''
        #Nothing needs to happen when the type is a standard schema element

    @classmethod
    def customize(cls, **kwargs):
        """
        This function duplicates and customizes the class it belongs to. The
        original class remains unchanged. This is an ugly hack. If you know
        better, let us know.
        """

        cls_dict = {}

        for k in cls.__dict__:
            if not (k in ("__dict__", "__weakref__")):
                cls_dict[k] = cls.__dict__[k]

        class Attributes(cls.Attributes):
            pass

        cls_dict['Attributes'] = Attributes

        class Annotations(cls.Annotations):
            pass

        cls_dict['Annotations'] = Annotations

        if not ('_is_clone_of' in cls_dict):
            cls_dict['_is_clone_of'] = cls

        for k,v in kwargs.items():
            if k != "doc" :
                setattr(Attributes,k,v)
            else :
                setattr(Annotations, k, v)

        cls_dup = type(cls.__name__, cls.__bases__, cls_dict)
        return cls_dup

class Null(Base):
    @classmethod
    def to_parent_element(cls, value, tns, parent_elt, name='retval'):
        element = etree.SubElement(parent_elt, "{%s}%s" % (tns,name))
        element.set('{%s}nil' % namespaces.ns_xsi, 'true')

    @classmethod
    def from_xml(cls, element):
        return None

class SimpleType(Base):
    __namespace__ = "http://www.w3.org/2001/XMLSchema"
    __base_type__ = None

    class Attributes(Base.Attributes):
        values = set()

    def __new__(cls, **kwargs):
        """
        Overriden so that any attempt to instantiate a primitive will return a
        customized class instead of an instance.

        See type.base.Base for more information.
        """

        retval = cls.customize(**kwargs)

        if not retval.is_default(retval):
            retval.__base_type__ = cls
            if retval.__type_name__ is None:
                retval.__type_name__ = kwargs.get("type_name", Base.Empty)

        return retval

    @staticmethod
    def is_default(cls):
        return (cls.Attributes.values == SimpleType.Attributes.values)

    @classmethod
    def get_restriction_tag(cls, schema_entries):
        simple_type = etree.Element('{%s}simpleType' % namespaces.ns_xsd)
        simple_type.set('name', cls.get_type_name())
        schema_entries.add_simple_type(cls, simple_type)

        restriction = etree.SubElement(simple_type,
                                            '{%s}restriction' % namespaces.ns_xsd)
        restriction.set('base', cls.__base_type__.get_type_name_ns(
                                                            schema_entries.app))

        for v in cls.Attributes.values:
            enumeration = etree.SubElement(restriction,
                                            '{%s}enumeration' % namespaces.ns_xsd)
            enumeration.set('value', str(v))

        return restriction

    @classmethod
    def add_to_schema(cls, schema_entries):
        if not schema_entries.has_class(cls) and not cls.is_default(cls):
            cls.get_restriction_tag(schema_entries)



#-*- coding: utf-8 -*-
# wsgi.py

"""Embed WSGI applications within Django projects.

An instance of `WSGIHandler` is an "application object" as defined in PEP 3333.
The recommended way to deploy a Django project as a WSGI application is::

    # django.wsgi

    application = django.core.handlers.wsgi.WSGIHandler()

The point of this module is to provide the opposite: deploy a WSGI application
in a Django project. This can be achieved either with a view wrapper or with a
middleware.

The view wrapper provides a straightforward integration in the URLconf::

    # urls.py

    from django.conf.urls.defaults import patterns, url
    from wsgiutil import embed_wsgi

    urlpatterns = patterns('',
        url(r'^my_wsgi_app(/.*)', embed_wsgi('my_wsgi_app.wsgi_callable')),
    )

`embed_wsgi` accepts either a WSGI callable or a path to the callable.

If `django.middleware.csrf.CsrfViewMiddleware` is enabled, you must wrap
`embed_wsgi` in `django.views.decorators.csrf.csrf_exempt`.

WSGI inherits from CGI the concepts of "path to the application object"
(`SCRIPT_NAME`) and "path within the application object" (`PATH_INFO`).
`SCRIPT_NAME` is defined by the first capture in the URL, if there is one, and
`PATH_INFO` is adjusted accordingly.

This setup does not provide a transparent integration, because Django will run
the middlewares. `django.middleware.csrf.CsrfViewMiddleware` will reject all
POST requests by default.

Using `EmbedWsgiMiddleware` to dispatch requests to the WSGI application gives
control on which middlewares will run, but it requires moving some routing
information outside of the URLconf::

    # settings.py

    MIDDLEWARE_CLASSES = (
        'wsgiutil.EmbedWsgiMiddleware',
        ...
    )

    EMBED_WSGI_URLS = (
        ('my_wsgi_app', 'my_wsgi_app.wsgi_callable'),
        ...
    )

The prefix in `EMBED_WSGI_URLS` is appended to `SCRIPT_NAME`, and `PATH_INFO`
is adjusted accordingly.

We will assume that Django is deployed with `WSGIHandler`, which means that
views receive instances of `WSGIRequest`, and that Django's implementation of
WSGI is PEP 3333 compliant. If that is not true, that is a bug in Django, and
this module will not attempt to fix it. Still, it will not a PEP 3333 compliant
WSGI gateway, because Django will always buffer the response.

`validate_wsgi` is an alternative for `embed_wsgi` that will raise errors on
violations of the WSGI specification.
"""


from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, MiddlewareNotUsed
from django.http import HttpResponse
from importlib import import_module
from wsgiref.validate import validator



class EmbedWsgiMiddleware(object):

    def __init__(self):
        try:
            embed_wsgi_urls = settings.EMBED_WSGI_URLS
        except AttributeError:
            raise ImproperlyConfigured('You must specify EMBED_WSGI_URLS in '
                                       'your settings file to use '
                                       'EmbedWsgiMiddleware.')
        if not embed_wsgi_urls:
            raise MiddlewareNotUsed('EMBED_WSGI_URLS is emtpy.')
        self.apps = {}
        for prefix, application in embed_wsgi_urls:
            self.apps['/' + prefix] = embed_wsgi(application)

    def process_request(self, request):
        for prefix, view in self.apps.iteritems():
            if request.path.startswith(prefix):
                shift_path(request.environ, prefix)
                return view(request)


def embed_wsgi(application):

    application = load_application(application)

    def view(request, *args, **kwargs):
        response = HttpResponse()

        # Enforce HTTPS if required
        if getattr(settings, 'SOAP_SERVER_HTTPS', True): #pragma: nocover
            if not request.is_secure() and not settings.DEBUG: # pragma: nocover 
                response.status_code = 400 # Bad request - we need https
                return response
        
        # Enforce basic auth
        if hasattr(settings, 'SOAP_SERVER_BASICAUTH_REALM'):
            if not 'HTTP_AUTHORIZATION' in request.environ:
                realm = getattr(settings, 'SOAP_SERVER_BASICAUTH_REALM', 'Webservice')
                response.status_code = 401 # Request auth
                response['WWW-Authenticate'] = 'Basic realm="%s"' % realm
                return response

        # request.environ and request.META are the same object, so changes
        # to the headers by middlewares will be seen here.
        environ = request.environ.copy()
        if len(args) > 0:
            shift_path(environ, '/' + args[0])
        # Django converts SCRIPT_NAME and PATH_INFO to unicode in WSGIRequest.
        environ['SCRIPT_NAME'] = environ.get('SCRIPT_NAME', '').encode('iso-8859-1')
        environ['PATH_INFO'] = environ.get('PATH_INFO', '').encode('iso-8859-1')

        headers_set = []
        headers_sent = []

        def write(data):
            if not headers_set:
                raise AssertionError("write() called before start_response()")
            if not headers_sent:
                # Send headers before the first output.
                for k, v in headers_set:
                    response[k] = v
                headers_sent[:] = [True]
            response.write(data)
            # We could call response.flush() here, but is actually a no-op.

        def start_response(status, headers, exc_info=None):
            # Let Django handle all errors.
            if exc_info:
                raise exc_info[1].with_traceback(exc_info[2])
            if headers_set:
                raise AssertionError("start_response() called again "
                                     "without exc_info")
            response.status_code = int(status.split(' ', 1)[0])
            headers_set[:] = headers
            # Django provides no way to set the reason phrase (#12747).
            return write

        result = application(environ, start_response)
        try:
            for data in result:
                if data:
                    write(data)
            if not headers_sent:
                write('')
        finally:
            if hasattr(result, 'close'):
                result.close()

        return response

    return view


def load_application(application):
    if isinstance(application, str):
        app_module, app_name = application.rsplit('.', 1)
        application = getattr(import_module(app_module), app_name)
    if not callable(application):
        raise ImproperlyConfigured('%s isn\'t callable' % application)
    return application


def shift_path(environ, prefix):
    assert environ['PATH_INFO'].startswith(prefix)
    environ['SCRIPT_NAME'] += prefix
    environ['PATH_INFO'] = environ['PATH_INFO'][len(prefix):]


def validate_wsgi(application):
    return validator(embed_wsgi(application))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 09:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('pcart_catalog', '0005_auto_20170220_2047'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductImageDownloadLink',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('url', models.TextField(verbose_name='URL')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('image', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='download_link', to='pcart_catalog.ProductImage', verbose_name='Image')),
            ],
            options={
                'verbose_name_plural': 'Product image download links',
                'verbose_name': 'Product image download link',
            },
        ),
        migrations.CreateModel(
            name='ProductImportLogMessage',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('message', models.TextField(blank=True, default='', verbose_name='Message')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
            ],
            options={
                'verbose_name_plural': 'Product import log messages',
                'verbose_name': 'Product import log message',
            },
        ),
        migrations.CreateModel(
            name='ProductTypeRelation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('alternate_uuid', models.UUIDField(help_text='UUID of the product category in 1C system.', verbose_name='UUID in 1C')),
                ('collections', models.ManyToManyField(blank=True, help_text='Add the products of specified type to a collection.', related_name='product_type_relations', to='pcart_catalog.Collection', verbose_name='Collections')),
                ('product_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relations', to='pcart_catalog.ProductType', verbose_name='ProductType')),
            ],
            options={
                'verbose_name_plural': 'Product type relations',
                'verbose_name': 'Product type relation',
            },
        ),
        migrations.CreateModel(
            name='XMLFile',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('filename', models.CharField(max_length=255, verbose_name='File name')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
            ],
            options={
                'verbose_name_plural': 'XML import files',
                'verbose_name': 'XML import file',
            },
        ),
        migrations.AddField(
            model_name='productimportlogmessage',
            name='file',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='import_log_messages', to='soap_1c_api.XMLFile', verbose_name='XML import file'),
        ),
        migrations.AddField(
            model_name='productimportlogmessage',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='import_log_messages', to='pcart_catalog.Product', verbose_name='Product'),
        ),
        migrations.AddField(
            model_name='productimportlogmessage',
            name='variant',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='import_log_messages', to='pcart_catalog.ProductVariant', verbose_name='Product variant'),
        ),
    ]



from celery import task
# from django.conf import settings


@task(bind=True)
def import_goods(self, xmlfile_id):
    from .lxml_import import import_goods_from_xml
    from .models import XMLFile

    try:
        t = XMLFile.objects.get(pk=xmlfile_id)
        self.update_state(state='PROGRESS')
        result = import_goods_from_xml(None, t.filename, task_instance=self)
        self.update_state(state='SUCCESS')
        return result
    except XMLFile.DoesNotExist as exc:
        raise self.retry(countdown=10, max_retries=2, exc=exc)



#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from soaplib.core.model.primitive import String
from soaplib.core.service import soap, DefinitionBase

class S1(DefinitionBase):
    name = 'S1Fools'
    __namespace__ = 'Hippity'

    @soap(String, _returns=String)
    def echo_string_s1(self, string):
        return string


class S2(DefinitionBase):
    name = 'S2Fools'
    __namespace__ = 'Hoppity'

    @soap(String, _returns=String)
    def bobs(self, string):
        return string 


class S3(DefinitionBase):

    name = 'S3Fools'
    __namespace__ = 'Hoppity'
    __service_interface__ = 'BLAHHHHAHHHAHHHHA'
    __port_types__ = ['bobhope', 'larry']


    @soap(String, _returns=String)
    def echo(self, string):
        return string

    @soap(String, _port_type='bobhope', _returns=String)
    def echo_bob_hope(self,  string):
        return 'Bob Hope'



class MissingRPCPortService(DefinitionBase):

    name = 'MissingRPCPortService'
    __namespace__ = 'MissingRPCPortService'
    __service_interface__ = 'MissingRPCPortService'
    __port_types__ = ['existing']

    @soap(String, _returns=String)
    def raise_exception(self, string):
        return string

class BadRPCPortService(DefinitionBase):
    name = 'MissingRPCPortService'
    __namespace__ = 'MissingRPCPortService'
    __service_interface__ = 'MissingRPCPortService'
    __port_types__ = ['existing']

    @soap(String,_port_type='existingss', _returns=String)
    def raise_exception(self, string):
        return string

#MissingServicePortService
class MissingServicePortService(DefinitionBase):
    name = 'MissingRPCPortService'
    __namespace__ = 'MissingRPCPortService'
    __service_interface__ = 'MissingRPCPortService'
    __port_types__ = ['existing']

    @soap(String,_port_type='existingss', _returns=String)
    def raise_exception(self, string):
        return string


class SinglePortService(DefinitionBase):
    name = 'SinglePort'
    __service_interface__ = 'SinglePortService_ServiceInterface'
    __namespace__ = 'SinglePortNS'
    __port_types__ = ['FirstPortType']

    @soap(String, _port_type='FirstPortType', _returns=String)
    def echo_default_port_service(self, string):
        return string


class DoublePortService(DefinitionBase):

    name = 'DoublePort'
    __namespace__ = 'DoublePort'
    __port_types__ = ['FirstPort', 'SecondPort']

    @soap(String, _port_type='FirstPort', _returns=String)
    def echo_first_port(self, string):
        return string

    @soap(String,_port_type='SecondPort', _returns=String)
    def echo_second_port(self, string):
        return string


#-*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt
from soap_server.webservice import DemoWebService
from soap_server.soap import Application
from django.conf import settings
'''
Since the list of applications will end up in the same namespace, make sure you 
don't have two types or methods called the same!

Notice how we must wrap the resulting view in a csrf_exempt() call? 
This is absolutely necessary since it would otherwise diverge from the RFC and 
thus would require custom webservice clients. 
'''

SOAP_SERVER_DEMO = getattr(settings, 'SOAP_SERVER_DEMO', True)

# This is just for readability's sake
application_view = Application([DemoWebService], 'ws', name='ws').as_django_view()

urlpatterns = patterns('')

if SOAP_SERVER_DEMO:
    urlpatterns += patterns('',
        url(r'^demo/', csrf_exempt(application_view))
    )



#!/usr/bin/env python
#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import unittest

from soaplib.core import namespaces

_ns_xs = namespaces.ns_xsd
_ns_xsi = namespaces.ns_xsi

from soaplib.core import Application
Application.transport = 'test'

from soaplib.core.service import DefinitionBase
from soaplib.core.service import soap

from soaplib.core.model.enum import Enum

from lxml import etree

vals = [
    'Monday',
    'Tuesday',
    'Wednesday',
    'Friday',
    'Saturday',
    'Sunday',
]

DaysOfWeekEnum = Enum(
    'Monday',
    'Tuesday',
    'Wednesday',
    'Friday',
    'Saturday',
    'Sunday',
    type_name = 'DaysOfWeekEnum'
)

class TestService(DefinitionBase):
    @soap(DaysOfWeekEnum, _returns=DaysOfWeekEnum)
    def remote_call(self, day):
        return DaysOfWeekEnum.Sunday

class TestEnum(unittest.TestCase):
    def test_wsdl(self):
        app = Application([TestService],'tns')
        wsdl = app.get_wsdl('punk')
        elt = etree.fromstring(wsdl)
        simple_type = elt.xpath('//xs:simpleType', namespaces=app.nsmap)[0]

        # Avoid printing debug output during test runs.
        #print etree.tostring(elt, pretty_print=True)
        #print simple_type

        self.assertEquals(simple_type.attrib['name'], 'DaysOfWeekEnum')
        self.assertEquals(simple_type[0].tag, "{%s}restriction" % namespaces.ns_xsd)
        self.assertEquals([e.attrib['value'] for e in simple_type[0]], vals)

    def test_serialize(self):
        DaysOfWeekEnum.resolve_namespace(DaysOfWeekEnum, 'punk')
        mo = DaysOfWeekEnum.Monday
        # Avoid printing debug output during test runs.
        #print repr(mo)

        elt = etree.Element('test')
        DaysOfWeekEnum.to_parent_element(mo, 'test_namespace', elt)
        elt = elt[0]
        ret = DaysOfWeekEnum.from_xml(elt)

        self.assertEquals(mo, ret)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 12:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0005_auto_20170220_2047'),
        ('soap_1c_api', '0002_auto_20170308_1305'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='producttyperelation',
            name='collections',
        ),
        migrations.AddField(
            model_name='producttyperelation',
            name='collection',
            field=models.OneToOneField(default=None, help_text='Add the products of specified type to a collection.', on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relation', to='pcart_catalog.Collection', verbose_name='Collections'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='producttyperelation',
            name='product_type',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relation', to='pcart_catalog.ProductType', verbose_name='Product type'),
        ),
    ]



"""Utility classes and methods for converting and validating
soaplib.core.model.ClassModel instances
"""

import os
import re
from lxml import etree

class ClassModelConverter():
    """A class to handle exporting a ClassModel to different representations

    Currently supported export targets are lxml.etree.Element, string and
    xml documents.

    This functionality will most likely be moved into the ClassModel itself if
    it proves useful and there is a willingness to modify the current
    ClassModel API.
    """

    def __init__(self, model_instance, tns, include_parent=False, parent_tag="root", include_ns=True):
        """
        @param An instance of a soaplib.core.model.clazz.ClassModel
        @parma The target namespace of the model instance.
        @param Indicates if a parent element should be returned as the root
        element of the xml representation.  If true, a root element will be included with
        the tag "parent_tag"
        @param The tag used for the creation of a root/parent element.
        """

        assert tns or tns !="" , "'tns' should not be None or an empty string"


        self.instance = model_instance
        self.tns = tns
        self.include_parent= include_parent
        self.parent_tag = parent_tag
        self.include_ns = include_ns


    def __get_ns_free_element(self, element):
        """ """

        new_el = None
        m = re.search('(?<=})\w+', element.tag)

        if m:
            new_el = etree.Element(m.group(0))
        else:
            new_el = etree.Element(element.tag)

        new_el.text = element.text

        for k in element.attrib.keys():
            if k not in ["{http://www.w3.org/2001/XMLSchema-instance}nil"]:
                new_el.attrib[k] = element.attrib[k]

        for child in element.iterchildren():
            new_child = self.__get_ns_free_element(child)
            new_el.append(new_child)

        return new_el


    def __get_etree(self):
        root_element = etree.Element(self.parent_tag)
        self.instance.to_parent_element(self.instance, self.tns, root_element)

        rt_el = None
        if not self.include_parent:
            rt_el = root_element[0]
        else:
           rt_el = root_element

        if not self.include_ns:
            rt_el = self.__get_ns_free_element(rt_el)

        return rt_el


    def to_etree(self):
        """Returns a lxml.etree.Element from a soaplib.core.model.clazz.ClassModel
        instance.
        """

        return self.__get_etree()


    def to_xml(self):
        """Returns a xml string from a soaplib.core.model.clazz.ClassModel instance.
        """

        el = self.to_etree()

        return etree.tostring(
                    el,
                    pretty_print=True,
                    encoding="UTF-8",
                    xml_declaration=True
                    )


    def to_file(self, file_path):
        """Writes a model instance to a XML document

        @param The output file path for the xml file
        """

        el = self.to_etree()

        f = open(file_path, "w")

        etree.ElementTree(el).write(
            f,
            pretty_print=True,
            encoding="UTF-8",
            xml_declaration=True
            )

        f.close()


#-*- coding: utf-8 -*-
"""
This file holds all the utils functions and helpers for the webservice.
"""
from django.contrib.auth import authenticate
from django.utils.log import logger
import base64

#===============================================================================
# Helpers
#===============================================================================

def get_user_from_environment(environ):
    '''
    This logs in a Django user from a Basic auth
    '''
    auth = environ.get('HTTP_AUTHORIZATION', None)
    assert auth != None, 'Webservice cannot be called without basic auth!'

    _, based = auth.split(' ') # typical string is 'Basic <b64 encoding>'
    plain_auth = base64.b64decode(based) # has the form '<username>:<password>'
    username, password = plain_auth.split(':')
    user = authenticate(username=username, password=password)
    return user




"""Mix-In Class to Export Decorated Methods using SOAP

   This file is not a Zope2 Product nor a Zope3 component.  It is a simple
   Python module that adds two elements: a hook into the Zope publisher to
   intercept SOAP requests, and a mix-in class to your Zope2 folder classes
   that make them SOAP-aware, with queryable WSDL descriptions of any methods
   decorated with @soapmethod.

   A tiny bit of code needs to be invoked from within ZPublisher during the
   parsing of an HTTP request that is a SOAP request.  To make this happen,
   change the code in the file lib/python/ZPublisher/HTTPRequest.py, within
   the processInputs() method of the HTTPRequest class, from this:

    fs=FieldStorage(fp=fp,environ=environ,keep_blank_values=1)
    if not hasattr(fs,'list') or fs.list is None:
        # Hm, maybe it's an XML-RPC
        if (fs.headers.has_key('content-type') and

   to this:

    fs=FieldStorage(fp=fp,environ=environ,keep_blank_values=1)
    if not hasattr(fs,'list') or fs.list is None:
        if environ.has_key('HTTP_SOAPACTION'):        #ADDED
            other['SOAPXML'] = fs.value               #ADDED
        # Hm, maybe it's an XML-RPC
        elif (fs.headers.has_key('content-type') and  #CHANGED
"""
import cgi

from xml.etree.ElementTree import ElementTree
from lxml import etree
from soaplib.core._base import MethodContext
from soaplib.core.mime import collapse_swa
from soaplib.core.model.exception import Fault

from soaplib.core.service import DefinitionBase,soap
from soaplib.core.model.primitive import string_encoding
from soaplib.core import Application
from soaplib.core.server._base import Base as BaseServer

from zope.interface import implements
from zope.interface.common.interfaces import IException
from zope.app.testing import ztapi

# public sumbols
__all__ = [
    'SoapFolder',  # to mix into your Zope Folders to make them into SOAP Service Points
    'AccessDeniedSOAP',  # exception object to signal a failed SOAP call
    ]


class SoapFolder(DefinitionBase):
    """Mix-In Class to Make a Folder into a SOAP Service Point

    Import this class into your Zope2 folder classes to make them SOAP-aware.
    Any methods in your folder class decorated with @soapmethod() will become
    callable over SOAP and the signature and docstring of that method will be
    reported as WSDL by a call to the index_html() method of that folder.

    Your class should also define a class attribute indicating the 'toplevel
    namespace' of your SOAP Service Point.  This name is arbitrary as far as
    this code goes, but appears in the SOAP response and the WSDL description
    generated.  This attribute looks like:   __tns__ = "PatientServices"
    """

    _v_soap_methods = None
    _v_cached_wsdl = None
    __wsdl__ = None


    def __init__(self, tns, environ=None):
        super(DefinitionBase, self).__init__(environ)
        self.soap_app = Application(self, tns, False)
        self.soap_handler = BaseServer(self.soap_app)


    def methods(self):
        """Returns a list of method descriptors for this object"""
        if self._v_soap_methods is None:
            self._v_soap_methods = self.build_public_methods()
        return self._v_soap_methods


    def service_description(self, REQUEST, RESPONSE):
        """ """

        if getattr(self, '__tns__', None) is None:
            self.__tns__ = self.get_tns(self.__class__)

        if self._v_soap_methods is None:
            self._v_soap_methods = self.build_public_methods()

        if self._v_cached_wsdl is None:
            self._v_cached_wsdl = self.soap_app.get_wsdl(self.absolute_url())
            self.__wsdl__ = None

        RESPONSE.setHeader('Content-Type', 'text/xml')
        return self._v_cached_wsdl

    def index_html(self, REQUEST, RESPONSE):
        """Handle an incoming SOAP request or a non-SOAP WSDL query."""

        if REQUEST.get('SOAPXML', None) == None:  # Not a SOAP Request, return WSDL
            return self.service_description(REQUEST, RESPONSE)

        try:
            # Deserialize the Body of the SOAP Request
            from soaplib.core._base import _from_soap
            header, payload = _from_soap(REQUEST.SOAPXML)

            # TODO: At this point I need dispatch method calls to the soaplib.Application
            # somehow....... :)
            ctx = MethodContext()

            content_type  = cgi.parse_header(REQUEST.get("Content-Type"))
            charset = content_type[1].get('charset',None)
            length = REQUEST.get("Content-Length")
            http_payload = REQUEST.read(int(length))

            if not charset:
                charset = "ascii"

            in_string = collapse_swa(content_type, http_payload)
            in_obj = self.soap_handler.get_in_object(ctx, in_string, charset)
            out_obj = self.soap_handler.get_out_object(ctx, in_obj)
            out_string = self.soap_handler.get_out_string(ctx, out_obj)
            
            return out_string

        except Exception, e:

            fault = Fault(faultstring=str(e))
            resp = etree.tostring(fault, encoding=string_encoding)

            RESPONSE.setStatus('InternalServerError', reason=faultstring)
            RESPONSE.setHeader('Content-Type', 'text/xml')
            return resp


class ISOAPException(IException):
    pass


class SOAPException(Exception):
    """Base exception class for all derived exceptions for SOAP"""

    implements(ISOAPException)

    def __init__(self, request):
        self.request = request
        self.request['faultexc'] = self

    def __str__(self):
        return self.__class__.__name__


class AccessDeniedSOAP(SOAPException):
    """An exception to raise in a SOAP method if access is being denied."""


class SOAPExceptionView:
    """Adapts an (ISOAPException, IRequest) to a View

       This view provides the XML representation of a SOAP fault that is
       returned to a caller.  To use it, register this view with Zope at some
       initialization point:

           from zope.app.testing import ztapi
           from dummy import ISOAPException, SOAPExceptionView
           ztapi.browserView(ISOAPException, u'index.html', SOAPExceptionView)

       and then within your SOAP logic raise a SOAP exception where needed:

           from dummy import SOAPException
           raise SOAPException(request)
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        faultstring = self.request['faultexc'].__class__.__name__
        self.request.response.setStatus('InternalServerError', reason=faultstring)

        faultcode = 'Server'
        fault = make_soap_fault(faultstring, faultcode, detail=None)

        self.request.response.setHeader('Content-Type', 'text/xml')
        return ElementTree.tostring(fault, encoding=string_encoding)

# The following registers 'SOAPExceptionView' as an adapter that knows how to
# display (generate and return the XML source for a SOAP fault) for anything
# that implements the 'ISOAPException' interface.

ztapi.browserView(ISOAPException, u'index.html', SOAPExceptionView)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 11:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('soap_1c_api', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='producttyperelation',
            name='alternate_uuid',
            field=models.UUIDField(help_text='UUID of the product category in 1C system.', unique=True, verbose_name='UUID in 1C'),
        ),
        migrations.AlterField(
            model_name='producttyperelation',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relations', to='pcart_catalog.ProductType', verbose_name='Product type'),
        ),
    ]




#!/usr/bin/env python
#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import unittest
import os
from datetime import datetime
from lxml import etree

from soaplib.core.model.clazz import ClassModel
from soaplib.core.model.primitive import Integer, String, DateTime
from soaplib.core.util.model_utils import ClassModelConverter

class SimpleModel(ClassModel):

    __type_name__ = "simplemodel"
    simple_text = String
    simple_num = Integer
    simple_date = DateTime

def simple_factory():
    simple = SimpleModel()
    simple.simple_text = "Text"
    simple.simple_num = 1234
    simple.simple_date = datetime(2001, 12, 12)
    return simple


class ComplexModel(ClassModel):
    __type_name__ = "complexmodel"
    simple = SimpleModel
    complex_text = String
    complex_num = Integer
    complex_date = DateTime


def complex_factory():
    simple = simple_factory()
    complex = ComplexModel()
    complex.simple = simple
    complex.complex_text = "ComplexText"
    complex.complex_num = 2222
    complex.complex_date = datetime(2010, 1, 1)
    return complex




class BaseCase(unittest.TestCase):

    def setUp(self):
        self.file_path = "instance.xml"
        self.converter = ClassModelConverter(simple_factory(), "tns")

    def tearDown(self):
        if os.path.isfile(self.file_path):
            os.unlink(self.file_path)

    def xml(self):
        xml = self.converter.to_xml()
        self.assertTrue(xml)

    def file(self):
        self.converter.to_file(self.file_path)
        self.assertTrue(os.path.isfile(self.file_path))

    def element(self):
        element =  self.converter.to_etree()
        self.assertTrue(element)
        tns_tag = '{%s}%s' % \
                  (self.converter.tns, self.converter.instance.__type_name__)
        self.assertEquals(element.tag, tns_tag)

    def remove_ns(self):
        self.converter.include_ns = False
        element = self.converter.to_etree()
        self.assertFalse(element.nsmap)

    def empty_ns(self):
        self.assertRaises(AssertionError, ClassModelConverter, simple_factory(), "")


class ModelAsRootTestCase(BaseCase):

    def test_simple_xml(self):
        self.xml()

    def test_simple_file(self):
        self.file()

    def test_simple_element(self):
        self.element()

    def test_complex_xml(self):
        self.xml()

    def test_complex_file(self):
        self.file()

    def test_complex_element(self):
        self.element()
        
    def test_strip_ns(self):
        self.remove_ns()

    def test_empty_ns(self):
        self.empty_ns()

class AddedRootElementTestCase(BaseCase):
    def setUp(self):
        self.file_path = "instance.xml"
        self.converter = ClassModelConverter(
                simple_factory(),"tns",include_parent=True, parent_tag="foo")

    def element(self):
        element =  self.converter.to_etree()
        self.assertTrue(element)
        self.assertEquals(element.tag, self.converter.parent_tag)

    def test_simple_xml(self):
        self.xml()

    def test_simple_file(self):
        self.file()

    def test_simple_element(self):
        self.element()

    def test_complex_xml(self):
        self.xml()

    def test_complex_file(self):
        self.file()

    def test_complex_element(self):
        self.element()

    def test_strip_ns(self):
        self.remove_ns()




#
# soaplib - Copyright (C) Soaplib contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from soaplib.core import namespaces
from soaplib.core import Application



def build_app(service_list, tns, name):

    app = Application(service_list, tns, name)
    app.transport = 'http://schemas.xmlsoap.org/soap/http'
    return app


class AppTestWrapper():

    def __init__(self, application):

        self.url = 'http:/localhost:7789/wsdl'
        self.service_string = '{%s}service' % namespaces.ns_wsdl
        self.port_string = '{%s}port' % namespaces.ns_wsdl
        self.soap_binding_string = '{%s}binding' % namespaces.ns_soap
        self.operation_string = '{%s}operation' % namespaces.ns_wsdl
        self.port_type_string = '{%s}portType' % namespaces.ns_wsdl
        self.binding_string = '{%s}binding' % namespaces.ns_wsdl

        self.app = application
        self.wsdl = self.app.get_wsdl(self.url)

    def get_service_list(self):

        return [
            s for s in self.app.wsdl.elements.iterfind(self.service_string)
        ]

    def get_port_list(self, service):

        return [p for p in service.iterfind(self.port_string)]

    def get_soap_bindings(self, binding):

        return [sb for sb in binding.iterfind(self.soap_binding_string)]

    def get_port_types(self):

        return [
            el for el in self.app.wsdl.elements.iterfind(self.port_type_string)
        ]

    def get_port_operations(self, port_type):

        return [o for o in port_type.iterfind(self.operation_string)]

    def get_bindings(self):

        return [
            el for el in self.app.wsdl.elements.iterfind(self.binding_string)
        ]

    def get_binding_operations(self, binding):
        return [o for o in binding.iterfind(self.operation_string)]
  


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-09 09:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('soap_1c_api', '0004_auto_20170308_1514'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='productimagedownloadlink',
            name='image',
        ),
        migrations.DeleteModel(
            name='ProductImageDownloadLink',
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class PCartImport1CConfig(AppConfig):
    name = 'soap_1c_api'
    verbose_name = _('Integration with 1C (SOAP)')



from django.contrib import admin
# from django.conf import settings
from .models import (
    XMLFile,
    ProductImportLogMessage,
    ProductTypeRelation,
    ExtraCollectionRelation,
)
# from django.utils.translation import ugettext as _


class XMLFileAdmin(admin.ModelAdmin):
    list_display = ('filename', 'added')
    search_fields = ('filename',)
    date_hierarchy = 'added'


admin.site.register(XMLFile, XMLFileAdmin)


class ProductImportLogMessageAdmin(admin.ModelAdmin):
    list_display = ('product', 'variant', 'file', 'message', 'added')
    raw_id_fields = ('product', 'variant', 'file')
    date_hierarchy = 'added'


admin.site.register(ProductImportLogMessage, ProductImportLogMessageAdmin)


class ProductTypeRelationAdmin(admin.ModelAdmin):
    list_display = (
        'title', 'alternate_uuid',
        'product_type', 'collection')

admin.site.register(ProductTypeRelation, ProductTypeRelationAdmin)


class ExtraCollectionRelationAdmin(admin.ModelAdmin):
    list_display = ('tag_name', 'collection')

admin.site.register(ExtraCollectionRelation, ExtraCollectionRelationAdmin)
